/******************************************************************************
 *                    Internetting Cooperating Programmers
 * ----------------------------------------------------------------------------
 *
 *  ____    PROJECT
 * |  _ \  __ _ _ __   ___ ___ _ __ 
 * | | | |/ _` | '_ \ / __/ _ \ '__|
 * | |_| | (_| | | | | (_|  __/ |   
 * |____/ \__,_|_| |_|\___\___|_|   the IRC bot
 *
 * Main authors:
 *  - Bjrn Reese <breese@imada.ou.dk>
 *  - Daniel Stenberg <Daniel.Stenberg@sth.frontec.se>
 *
 * Contributing authors (and ports):
 *  - Philippe Brand <phb@telesys-innov.fr>
 *  - Bjrn Stenberg <Bjorn.Stenberg@sth.frontec.se>
 *
 * All files in this archive are subject to the GNU General Public License.
 * 
 *               HTTP://www.imada.ou.dk/~breese/dancer.html
 *                FTP://ftp.netcom.com/pub/ra/raydar/Dancer
 *
 * $Source: /home/hardcore/dast/dancer/src/RCS/bans.h,v $
 * $Revision: 2.2 $
 * $Date: 1997/11/24 23:32:53 $
 * $Author: dast $
 * $State: Exp $
 * $Locker:  $
 *
 * ---------------------------------------------------------------------------
 * $Log: bans.h,v $
 * Revision 2.2  1997/11/24 23:32:53  dast
 * added nick part to warn patterns
 *
 * Revision 2.1  1996/12/16 18:16:26  dast
 * GUESSNICK
 *
 * Revision 1.20  1996/12/13 18:10:05  dast
 * warnlist timeout after 3 months
 *
 * Revision 1.19  1996/12/06 15:54:30  dast
 * added two new protos
 *
 * Revision 1.18  1996/12/03 16:56:27  dast
 * added two missing prototypes
 *
 * Revision 1.17  1996/09/03 15:21:36  dast
 * Added kick message to the kick queue
 *
 * Revision 1.16  1996/08/26 07:11:45  dast
 * AddToBanlist() defined added
 *
 * Revision 1.15  1996/08/06 10:51:32  dast
 * Fixed header
 *
 * Revision 1.14  1996/07/20 16:10:56  dast
 * IllegalBan() prototype
 *
 * Revision 1.13  1996/07/19 08:42:55  dast
 * Added defines for AddToBanlist() return codes
 *
 * Revision 1.12  1996/07/15 12:44:32  dast
 * Added the two new defines BAN_SENTUNBAN and BAN_SENTBAN
 *
 * Revision 1.11  1996/07/10 16:15:00  dast
 * Added the source header.
 *
 *
 *****************************************************************************/
#ifndef __BANS_H
#define __BANS_H

typedef struct KickStruct {
  struct Header h;
  char *pattern;    /* user account */
  char *nick;       /* last used nick name */
  char *kickmsg;    /* reason */
  long kicks;       /* counter of kicks within KICK_PERIOD */
  long botkicks;    /* kicks by the bot */
  long trustkicks;  /* kicks when trusted user */
  long totkicks;    /* total count of kicks */
  time_t lastkick;  /* time of last kick */
  char *kicker;     /* last kicker */
  bool norecurse;   /* to prevent recursion of the same pattern in
		       the AddKick() function */
} itemkick;

extern itemkick* kickHead;

void BanKick(char *nick, char *banthis, char *message, char *remove);
void BanTimeout(void);

int AddToBanList(int flags, char *who, char *nick, char *pattern,
                  int secs, char *reason);
bool ChangeInBanList(char *who, char *bannick, int newsecs,
                     char *pattern, char flag, char *reason);
bool UnBan(char *from, char *nick);
void Unbanned(char *from, char *pattern, bool bot);
void BanInit(void);
bool IsUnban(char *pattern);
bool IsBanListed(char *pattern);
bool IsBan(char *pattern);
char *IsMatchBan(char *checkthis, int *);
void BanDisable(void);
bool IllegalBan(char *);
void BanSave(void);
void BanCleanup(void);
bool UnbanLoprio(char *from);

bool AddKick(itemident *, char *, char *, int);
void FreeKick(void *v);
void KickAll(char *pattern, char *msg);
void KickInit(void);
void KickCleanup(void);
void KickList(char *, char *);
bool KickFromQueue(time_t *);
itemkick *KickMatch(char *pattern);


void WarnFree(void * );
void WarnCleanup(void);
bool WarnAdd(char * , char *, char * , char * , char *, int );
void WarnSave(void);
bool WarnDel(char * , char * );
bool WarnList(char * , char * , int );
void WarnInit(void);
bool WarnCheck(char * , char *, char * );
int CountWarns(int);

#define BANLIST_ALERTSIZE 18 /* warn in public when we have this many, or
                                more, bans in the list */
#define REFRESH_TIMEOUT (30*SECINMIN)
#define SERVERBANTIMEOUT 30 /* give 30 secs to complete server bans */
#define BAN_ENFTIMEOUT (1<<0) /* timeouted enforced ban */
#define BAN_ENFORCE (1<<1) /* make this always present */
#define BAN_ACTUAL  (1<<2) /* this is guaranteed to be a valid channel-
                              ban == we got this from the banlist or from a
                              mode change within the channel */
#define BAN_UNBAN   (1<<3) /* this is unbanned */
#define BAN_UNSET   (1<<4) /* this is read from the file when bottie was
                              started and not yet set */
#define BAN_SENTUNBAN (1<<5) /* unban sent to server */
#define BAN_SENTBAN   (1<<6) /* ban sent to server */
#define BAN_GUESSNICK (1<<7) /* guessed the nick, may be wrong */
#define BAN_ALL     ~0     /* all flags */

/*
 * An easy-to-use define to see if the ban is there or about to pop up soon:
 */
#define BANISTHERE (BAN_ACTUAL|BAN_SENTBAN)

/* required level that unbans enforced bans just like any other ban */
#define BAN_ENFORCELEVEL LEVELBOT

#define BANRETRY_ENFORCE 30  /* number of minutes to wait until an enforced
                                ban is rebanned after unbanned twice rapidly
                              */

/* We don't save warnings to the warn file that were set and not warned for
   during this period of time: */
#define WARN_TIMEOUT_REMOVAL (SECINMONTH*3)

#define CHBAN_ENFORCE    1
#define CHBAN_UNENFORCE  2
#define CHBAN_REASON     3

#define KICK_TRUSTEDUSER 2
#define KICK_BOT         1
#define KICK_COMMON      0

#define WARNF_KICKBAN 1    /* kick user and ban pattern on sight */
#define WARNF_ALL     0xff /* all warn flags ORed */

#define WLIST_ALL 1 /* list all info */
#define WLIST_BAN 2 /* list only bans */

#define BLIST_ACTUAL 1   /* present and ACTUAL in the list */
#define BLIST_PRESENT -1 /* present in list, but not ACTUAL */
#define BLIST_NOTHERE 0  /* unbanned or not existing before */

#endif
