#ifndef __COMMAND_H
#define __COMMAND_H
/******************************************************************************
 *                    Internetting Cooperating Programmers
 * ----------------------------------------------------------------------------
 *
 *  ____    PROJECT
 * |  _ \  __ _ _ __   ___ ___ _ __ 
 * | | | |/ _` | '_ \ / __/ _ \ '__|
 * | |_| | (_| | | | | (_|  __/ |   
 * |____/ \__,_|_| |_|\___\___|_|   the IRC bot
 *
 * Main authors:
 *  - Bjrn Reese <breese@imada.ou.dk>
 *  - Daniel Stenberg <Daniel.Stenberg@sth.frontec.se>
 *
 * Contributing authors (and ports):
 *  - Philippe Brand <phb@telesys-innov.fr>
 *  - Bjrn Stenberg <Bjorn.Stenberg@sth.frontec.se>
 *
 * All files in this archive are subject to the GNU General Public License.
 * 
 *               HTTP://www.imada.ou.dk/~breese/dancer.html
 *                FTP://ftp.netcom.com/pub/ra/raydar/Dancer
 *
 * $Source: /home/hardcore/dast/dancer/src/RCS/command.h,v $
 * $Revision: 4.3 $
 * $Date: 1997/08/25 17:44:00 $
 * $Author: dast $
 * $State: Exp $
 * $Locker:  $
 *
 * ---------------------------------------------------------------------------
 * $Log: command.h,v $
 * Revision 4.3  1997/08/25 17:44:00  dast
 * fileget proto
 *
 * Revision 4.2  1997/07/15 07:41:21  dast
 * Conference mode proto and defines added
 *
 * Revision 4.1  1997/05/28 06:08:34  dast
 * Added the news prototypes
 *
 * Revision 3.3  1997/04/29 16:18:34  dast
 * new proto
 *
 * Revision 3.2  1997/04/28 16:36:39  dast
 * added new protos
 * change the help struct member
 *
 * Revision 3.1  1997/01/27 16:59:38  dast
 * added the remote field in the command struct
 * new proto
 *
 * Revision 2.3  1997/01/07 16:49:08  dast
 * new hide field in struct Command
 *
 * Revision 2.2  1997/01/05 21:06:47  dast
 * added proto
 *
 * Revision 2.1  1996/12/16 18:18:50  dast
 * add proto and sorted them
 *
 * Revision 1.20  1996/12/10 18:03:17  dast
 * added proto
 *
 * Revision 1.19  1996/12/09 18:13:44  dast
 * added two new command fields
 * added defines
 * added protos
 *
 * Revision 1.18  1996/12/03 18:51:19  dast
 * added proto
 *
 * Revision 1.17  1996/09/03 15:22:45  dast
 * changed the NickToPattern() proto
 *****************************************************************************/

#include "user.h"
#include "language.h"

struct Command {
  char *name;
  void (*function)(char *a, char *b);
  int level;
  bool showable;  /* Output can be redirected to another person */
  bool needpass;  /* Password protected */
  bool needcomma; /* Comma is part of argument, not argument seperator */
  bool aliases;   /* Aliases exist for this command */
  bool public;    /* Command can be used in public */
  bool hide;      /* Don't show the command line in the logfile or spylink */
  bool remote;    /* Can be executed by remote link */
  char *syntax;
  char **help;
  long flags;
  
  /* end of struct statically inited in command.c */

  long counter;
  long orglevel; /* Built-in level, never change, needed for internal use */
};

#define CMD_NONE 0 /* startup-flags */

#define CMD_CHANGED  (1<<0) /* command is changed */
#define CMD_IDENT    (1<<1) /* hide if IDENTPROTECT is enabled */
#define CMD_CONF     (1<<2) /* conference mode command */
#define CMD_CONFOPER ((1<<3)|CMD_CONF) /* CONFOPER command */
#define CMD_VICTIM   (1<<4) /* VICTIM command */

char *_crypt(char *key, char *salt);

void InitCommands();
struct Command *FindCommand(char *name);
int Warning(itemguest *, char *, char *);
void UpperCheck(itemguest *, char *);
void FloodCheck(itemguest *);
int NickToPattern(char *nick, char *buffer, bool siteban);

void Command(char *, char *);
void DccCommand(char *);
void PubCommand(char *, char *);
void ChgCommandLevel(char *, char *);

void CmdAlias(char *, char *);
void CmdBan(char *, char *);
void CmdBacklog(char *, char *);
void CmdBanlist(char *, char *);
void CmdBotop(char *, char *);
void CmdBug(char *, char *);
void CmdCalc(char *, char *);
void CmdChat(char *, char *);
void CmdChban(char *, char *);
void CmdCkey(char *, char *);
void CmdClient(char *, char *);
void CmdClientlist(char *, char *);
void CmdCmd(char *, char *);
void CmdCmode(char *, char *);
void CmdComment(char *, char *);
void CmdConf(char *, char *);
void CmdConvert(char *, char *);
void CmdCountry(char *, char *);
void CmdCut(char *, char *);
void CmdDebugadd(char *, char *);
void CmdDebugdel(char *, char *);
void CmdDefPass(char *, char *);
void CmdDeVoice(char *, char *);
void CmdDo(char *, char *);
void CmdEmail(char *from, char *line);
void CmdExchange(char *from, char *line);
void CmdExpAdd(char *, char *);
void CmdExpDel(char *from, char *line);
void CmdExplain(char *, char *);
void CmdFileGet(char *, char *);
void CmdFind(char *, char *);
void CmdFinger(char *, char *);
void CmdHelp(char *, char *);
void CmdHost(char *, char *);
void CmdIdent(char *, char *);
void CmdIdentify(char *, char *);
void CmdInvite(char *from, char *line);
void CmdJoin(char *, char *);
void CmdKick(char *, char *);
void CmdKicklist(char *, char *);
void CmdLag(char *, char *);
void CmdLanguage(char *, char *);
void CmdLeave(char *, char *);
void CmdLink(char *, char *);
void CmdLinklist(char *, char *);
void CmdLog(char *, char *);
void CmdMe(char *, char *);
void CmdModerate(char *, char *);
void CmdMute(char *, char *);
void CmdNames(char *, char *);
void CmdNewPass(char *, char *);

void CmdNewsRead(char *from, char *line);
void CmdNewsDel(char *from, char *line);
void CmdNewsAdd(char *from, char *line);
void CmdNewsList(char *from, char *line);

void CmdNick(char *, char *);
void CmdNone(char *, char *);
void CmdOp(char *, char *);
void CmdPass(char *, char *);
void CmdQuiet(char *, char *);
void CmdQuit(char *, char *);
void CmdRegister(char *, char *);
void CmdReload(char *, char *);
void CmdRemote(char *, char *);
void CmdReportadd(char *, char *);
void CmdReportdel(char *, char *);
void CmdRules(char *, char *);
void CmdSay(char *, char *);
void CmdSay(char *, char *);
void CmdSeen(char *, char *);
void CmdServer(char *, char *);
void CmdServerAdd(char *from, char *line);
void CmdServerDel(char *from, char *line);
void CmdServerList(char *from, char *line);
void CmdServerStat(char *from, char *line);
void CmdSet(char *, char *);
void CmdSetEmail(char *from, char *line);
void CmdShutdown(char *, char *);
void CmdSpell(char *, char *);
void CmdSplit(char *, char *);
void CmdSpyadd(char *, char *);
void CmdSpyadd(char *, char *);
void CmdSpydel(char *, char *);
void CmdSpydel(char *, char *);
void CmdSpylist(char *, char *);
void CmdStatus(char *, char *);
void CmdSview(char *, char *);
void CmdSyntax(char *, char *);
void CmdTalk(char *, char *);
void CmdTelladd(char *, char *);
void CmdTelldel(char *, char *);
void CmdTelllist(char *, char *);
void CmdTellme(char *, char *);
void CmdTopic(char *, char *);
void CmdToplist(char *, char *);
void CmdTz(char *, char *);
void CmdUnban(char *, char *);
void CmdUptime(char *, char *);
void CmdUsed(char *, char *);
void CmdUserAdd(char *from, char *line);
void CmdUserDel(char *from, char *line);
void CmdUserMod(char *from, char *line);
void CmdUserlist(char *, char *);
void CmdVoice(char *, char *);
void CmdVrfy(char *, char *);
void CmdWall(char *, char *);
void CmdWarnadd(char *, char *);
void CmdWarndel(char *, char *);
void CmdWarnlist(char *, char *);
void CmdWebster(char *, char *);
void CmdWhois(char *, char *);
void CmdWholeft(char *, char *);

void DispComment(itemguest *g);

char *VarInfo(int arg);
#endif
