#ifdef RCS
static char rcsid[]="$Id: ctcp.c,v 2.5 1998/02/14 11:32:48 dast Exp $";
#endif
/******************************************************************************
 *                    Internetting Cooperating Programmers
 * ----------------------------------------------------------------------------
 *
 * ____    PROJECT
 * |  _ \  __ _ _ __   ___ ___ _ __ 
 * | | | |/ _` | '_ \ / __/ _ \ '__|
 * | |_| | (_| | | | | (_|  __/ |   
 * |____/ \__,_|_| |_|\___\___|_|   the IRC bot
 *
 * Main authors:
 *  - Bjrn Reese <breese@imada.ou.dk>
 *  - Daniel Stenberg <Daniel.Stenberg@sth.frontec.se>
 *
 * Contributing autors (and ports):
 *  - Philippe Brand <phb@telesys-innov.fr>
 *  - Bjrn Stenberg <Bjrn.Stenberg@sth.frontec.se>
 *
 * All files in this archive are subject to the GNU General Public License.
 * 
 *               HTTP://www.imada.ou.dk/~breese/dancer.html
 *                FTP://ftp.netcom.com/pub/ra/raydar/Dancer
 *
 * $Source: /home/hardcore/dast/dancer/src/RCS/ctcp.c,v $
 * $Revision: 2.5 $
 * $Date: 1998/02/14 11:32:48 $
 * $Author: dast $
 * $State: Exp $
 * $Locker:  $
 *
 * ---------------------------------------------------------------------------
 * $Log: ctcp.c,v $
 * Revision 2.5  1998/02/14 11:32:48  dast
 * added boundary checks
 *
 * Revision 2.4  1998/01/26 17:19:56  dast
 * public CTCPs were not treated correctly
 *
 * Revision 2.3  1997/11/05 21:58:45  dast
 * bugfixed
 *
 * Revision 2.2  1997/04/28 16:37:53  dast
 * commented out a little
 *
 * Revision 2.1  1997/01/27 17:02:18  dast
 * now does quoting and multiple CTCPs (breese)
 *
 * Revision 1.23  1997/01/05 21:08:14  dast
 * less verbose ctcp flood logging
 * no replies to illegal CTCP requests
 *
 * Revision 1.22  1996/12/09 18:14:37  dast
 * uses the new FindUser()
 *
 * Revision 1.21  1996/11/04 19:15:21  dast
 * better HAVE_LIBFPL usage
 * breese fixes
 *
 * Revision 1.20  1996/09/05 06:01:49  dast
 * Added CTCP* and ACTION hooks
 *
 * Revision 1.19  1996/08/30 11:29:13  dast
 * uses the 'now' variable
 * prefix fixed
 *
 * Revision 1.18  1996/08/21 11:17:31  dast
 * breese's UDP multibot link
 *
 * Revision 1.17  1996/08/05 13:34:13  dast
 * breese's DCC modifies
 *
 * Revision 1.15  1996/07/26 11:34:58  dast
 * CTCP cleanup by breese
 *
 * Revision 1.14  1996/07/23 07:53:42  dast
 * changed atol to atoul for the OpenDccChat() call
 *
 * Revision 1.13  1996/07/18 13:51:50  dast
 * Flood checks are moved to flood.c
 *
 * Revision 1.12  1996/07/10 16:15:00  dast
 * Added the source header.
 *
 *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "dancer.h"
#include "function.h"
#include "user.h"
#include "transfer.h"
#include "flood.h"
#include "netstuff.h"
#include "ctcp.h"
#include "ibcp.h"
#include "fplrun.h"
#include "link.h"

/* --- Global ----------------------------------------------------- */

extern time_t now;

extern char channel[], ctcpversion[], ctcpuserinfo[];
extern bool avalance, uppercheck;
extern bool floodmode, xdccmode, repeatmode;
extern bool ctcpmode, shuttingdown;
extern bool public;
extern int serverport;

extern char nickname[];
extern char opaction[];
extern int opactionlen;
extern long levels[];

/* --- Private CTCPs --- */
struct
{
  char *name;
  void (*function)(char *a, char *b);
} ctcps[] =
{
  {"PING",       CtcpPing},
  {"DCC",        CtcpDCC},
  {"CLIENTINFO", CtcpClientinfo},
  {"VERSION",    CtcpVersion},
  {"TIME",       CtcpTime},
  {"USERINFO",   CtcpUserinfo},
  {NULL, (void(*)())(NULL)}
};

/* --- Public CTCPs --- */
struct
{
  char *name;
  void (*function)(char *a, char *b);
} pubctcps[] =
{
  {"ACTION",     CtcpAction},
  {"PING",       CtcpPing},
  {"CLIENTINFO", CtcpClientinfo},
  {"VERSION",    CtcpVersion},
  {"TIME",       CtcpTime},
  {"USERINFO",   CtcpUserinfo},
  {"XDCC",       CtcpXdcc},
  {NULL, (void(*)())(NULL)}
};


/* --- IgnoreCtcp ------------------------------------------------- */

  /*
   * This deals with private CTCP commands to the bot. CTCPFloodCheck()
   * takes care of the ones in the channel.
   */

time_t lastctcptime = NULL;
bool ctcpignore = FALSE;
int ctcpcount = 0;
int ctcpignorednum = 0;
time_t ctcpignorestart = NULL;

time_t ctcptimeout; /* the actual time-out, no more replies for a while */

bool IgnoreCtcp(char *from, char *cmd)
{
  /* Got more than 2 requests within 3 seconds? */
  if ( now - lastctcptime <= 3 ) {
    if (++ctcpcount > 2) {
      lastctcptime = now;
      if (!ctcpignore) {
        char buf[BIGBUFFER];
        strtok(cmd, "\001");
        sprintf(buf, "Flood: Ignoring %s flood by %s", cmd, from);
        Multicast(REPORTCAST, buf);
        Log(LOGCTCP, buf);
      }
      CTCPignore();
      ctcpignore = TRUE;
      return TRUE; /* Ignore request */
    }
  } else
    ctcpcount = 0;
  ctcpignore = FALSE;
  lastctcptime = now;
  return FALSE;
}


/* --- CTCP ------------------------------------------------------- */

void CTCP(char *from, char *line)
{
  register int i;
  char *cmd, *rest;
  char msg[BIGBUFFER];

  snapshot;
  public = FALSE;

  /* Remove trailing \001 and skip initial */
  /* Stacked requests aren't handled properly */
#if 0
  if ( (param = strrchr(line++, '\001')) == NULL )
    return;
  *param = NIL;
#endif

  CtcpUnquote(line, msg);
  rest = msg;

#ifdef HAVE_LIBFPL
  if(runfpl(RUN_CTCPPRIV, msg, FPLRUN_PRE))
    return;
#endif

  cmd = NextWord(rest);
  if (cmd) {
    if(now < ctcptimeout) {
      /*
       * Ignoring CTCPs.
       * But we can let through 20+ users' DCC chat requests!
       */
      char nick[13];
      itemguest *w;
      if(sscanf(from, "%*[^!]!%*[~^+-=]%[^@]", nick) &&
         (w = FindNick(nick)) &&
         w->ident->level>=LEVELCHANOP &&
         !strcasecmp("DCC", cmd))
        ; /* all this makes it go through the ignore */
      else {
        CTCPignore();
        /*Logf(LOGCTCP, "Ignores %s from %s", cmd, from);*/
        return; /* ignored */
      }
    }
    else if ( IgnoreCtcp(from, cmd) )
      return;
    if (ctcpignorednum > 0) {
      Logf(LOGCTCP, "Ignored %d CTCP requests in %s at the last flood",
	   ctcpignorednum,
	   TimeAgo(ctcptimeout-TIME_CTCPIGNORE-ctcpignorestart+now));
      ctcpignorednum = 0;
    }

    /* Handle normal private ctcp request */

    for (i=0; ctcps[i].name; i++) {
      if ( !strcasecmp(ctcps[i].name, cmd) ) {
        ctcps[i].function(from, rest);
        break;
      }
    }
  }
  runfpl(RUN_CTCPPRIV, msg, FPLRUN_POST);
}

void CTCPignore(void)
{
  if (ctcpignorednum == 0)
    ctcpignorestart = now;
  ctcptimeout = now + TIME_CTCPIGNORE;
  ctcpignorednum++;
}

/* --- PubCTCP ---------------------------------------------------- */

void PubCTCP(char *from, char *line)
{
  register int i;
  char *cmd, *param;
  char nik[NICKLEN+1];
  char msg[BIGBUFFER];
  itemguest *w;

  snapshot;
  public = TRUE;
#if 0 /* breese fix */
  /* Remove trailing \001 and skip initial */
  if ( (param = strrchr(line++, '\001')) == NULL )
    return;
  *param = NIL;
#endif
  CtcpUnquote(line, msg);

#ifdef HAVE_LIBFPL
  if(runfpl(RUN_CTCPPUB, msg, FPLRUN_PRE))
    return;
#endif

  if (1 <= sscanf(from, "%"NICKLENTXT"[^!]", nik) ) {

    w = FindNick(nik);
    if (w) {
      /* Handle public CTCP avalances */
      if (avalance)
	AvalanceCheck(w, msg);
    }
    /* Slower flood check on ACTION */
    if ((CTCPFloodCheck(w, !strncasecmp("ACTION", msg, 6))) ||
        (now < ctcptimeout)) {
      /* ignoring CTCPs */
      CTCPignore();
      /*Logf(LOGCTCP, "Ignores channel '%s' from %s", msg, from);*/
      return;
    }
    if (ctcpignorednum > 0) {
      Logf(LOGCTCP, "Ignored %d channel CTCP requests in %s at the last flood",
	   ctcpignorednum,
	   TimeAgo(ctcptimeout-TIME_CTCPIGNORE-ctcpignorestart+now));
      ctcpignorednum = 0;
    }

    cmd = strtok(msg, " ");
    if (cmd) {
      param = strtok(NULL, "");
      for (i=0; pubctcps[i].name; i++) {
        if ( !strcasecmp(pubctcps[i].name, cmd) ) {
          pubctcps[i].function(from, param);
          break;
        }
      }
    }
  }
  runfpl(RUN_CTCPPUB, msg, FPLRUN_POST);
}

/* --- CtcpXdcc --------------------------------------------------- */

void CtcpXdcc(char *from, char *msg)
{
  if (xdccmode) {
    itemguest *w;
    strtok(from, "!");
    w = FindNick(from);
    if (w)
      StickyKick(w, "xdcc this!");
  }
}

/* --- CtcpClientinfo --------------------------------------------- */

void CtcpClientinfo(char *from, char *msg)
{
  if (public)
    Logf(LOGCTCP, "Clientinfo on %s from %s", channel, from);
  else
    Logf(LOGCTCP, "Clientinfo from %s", from);
  strtok(from, "!");
  ReplyCtcpf(from, "CLIENTINFO %s", CLIENTINFOMSG);
}

/* --- CtcpPing --------------------------------------------------- */

void CtcpPing(char *from, char *msg)
{
  if (public)
    Logf(LOGCTCP, "Ping on %s from %s", channel, from);
  else
    Logf(LOGCTCP, "Ping from %s", from);
  strtok(from, "!");
  if (msg && *msg)
    ReplyCtcpf(from, "PING %s", msg);
  else
    ReplyCtcp(from, "ERRMSG Incorrect Ping request - time missing");
}

/* --- CtcpVersion ------------------------------------------------ */

void CtcpVersion(char *from, char *msg)
{
  if (public)
    Logf(LOGCTCP, "Version on %s from %s", channel, from);
  else
    Logf(LOGCTCP, "Version from %s", from);
  strtok(from, "!");
  ReplyCtcpf(from, "VERSION %s", ctcpversion);
}

/* --- CtcpUserinfo ----------------------------------------------- */

void CtcpUserinfo(char *from, char *msg)
{
  if (public)
    Logf(LOGCTCP, "Userinfo on %s from %s", channel, from);
  else
    Logf(LOGCTCP, "Userinfo from %s", from);
  strtok(from, "!");
  ReplyCtcpf(from, "USERINFO %s", ctcpuserinfo);
}

/* --- CtcpTime --------------------------------------------------- */

void CtcpTime(char *from, char *msg)
{
  time_t tid;
  char tiden[MIDBUFFER];

  if (public)
    Logf(LOGCTCP, "Time on %s from %s", channel, from);
  else
    Logf(LOGCTCP, "Time from %s", from);
  strtok(from, "!");
  if ( (tid = now) != -1) {
    strcpy(tiden, ctime(&tid));
    strtok(tiden, "\n"); /* ctime() appends a \n, get rid of it */
    ReplyCtcpf(from, "TIME %s", tiden);
  } else
    ReplyCtcp(from, "ERRMSG Unable to get systemtime");
}

/* --- CtcpAction ------------------------------------------------- */

void CtcpAction(char *from, char *msg)
{
  itemguest *ic;

  strtok(from, "!");
  ic = FindNick(from);

#ifdef HAVE_LIBFPL
  if(runfpl(RUN_ACTION, msg, FPLRUN_PRE))
    return;
#endif

  if (*opaction) {  /* A kind of passive auto-op */
    if (ic && ic->ident->user &&
        ic->ident->user->level >= LEVELBOT &&
	msg && *msg &&
        !strncasecmp(msg, opaction, opactionlen) )
      Mode("+o %s", from);
  }

  if (ic) {
    ic->posts++;

    Check(ic, msg);

    if(repeatmode)
      RepeatCheck(ic, msg);

    ic->posttime = now;
    Logf(LOGPUB, "* %s %s", from, msg ? msg : "");
  }
  runfpl(RUN_ACTION, msg, FPLRUN_POST);
}

/* --- OpenDccChat ------------------------------------------------ */

void OpenDccChat(char *from, char *uhost, ulong num, ushort port)
{
  itemuser *u;
  itemclient *k;

  if ( !(u=FindUser(from, uhost)) ) { /* Only allow registered users */
    ReplyCtcp(from, "ERRMSG No authentication");

  } else if ( (k=FindClientByNick(from)) ) { /* Only one client per nick */
    if (k->status == CL_CONNECTED)
      ReplyCtcp(from, "ERRMSG Another connection already exist");
    else
      ReplyCtcp(from, "ERRMSG Another attempt to connect exist. Use CUT to cancel that attempt");

  } else {
    OpenClientConnection(from, uhost, u, MakeIP(num), port);
  }
}

/* --- OpenUdpLink ------------------------------------------------ */

void OpenUdpLink(char *from, char *uhost, ulong num, ushort port)
{
  itemuser *u;
  itemlink *r;

  u = FindUser(from, uhost);
  if ( u && u->flags.linkbot )
    {
      r = AddLink(from, u, num, port);
      if (r)
	{
	  r->tmpvalue = Random();
	  SendLink(r, IBCP_HELLO, "%d %d %s %lu",
		   IBCP_VERSION, IBCP_BOT_DANCER, nickname, r->tmpvalue);
	}
      else
	ReplyCtcp(from, "ERRMSG Insufficient resources");
    }
  else
    ReplyCtcp(from, "ERRMSG No authentication");
}

/* --- CtcpDCC ---------------------------------------------------- */

void CtcpDCC(char *from, char *msg)
{
  char *uhost;
  ushort port;
  char type[5];
  char darg[BIGBUFFER];
  char snum[MIDBUFFER];
  char sport[MIDBUFFER];

  if (msg && *msg) {

    Logf(LOGCTCP, "DCC request from %s (%s)", from, msg);

    strtok(from, "!");
    uhost = strtok(NULL, "");

    if (shuttingdown) {
      ReplyCtcp(from, "ERRMSG Shutdown in progress. Cannot accept connections");
      return; /* Make sure we exit */

    }
    else if (4 != sscanf(msg, "%4s %"BIGBUFFERTXT"s %"MIDBUFFERTXT"s %"MIDBUFFERTXT"s",
			 type, darg, snum, sport) ) {
      ReplyCtcp(from, "ERRMSG Erroneous DCC request");
      Debug("Wrong number of DCC arguments");

    } else if ( (port=atol(sport)) == serverport ) {
      /* User sucks, don't even answer */
      Debug("%s (%s) attempted to DCC to serverport (%d)",
            from, uhost, port);

    } else if ( port < 1024 ) {
      ReplyCtcp(from, "ERRMSG Connection rejected. Privileged port");
      Debug("%s (%s) rejected - privileged port (%d)",
            from, uhost, port);

    } else {

      if ( !strcasecmp(type, "CHAT") ) {

        /* darg ought to be "chat", but some clients use username
	   instead. We'll just ignore it. */
	OpenDccChat(from, uhost, atoul(snum), port);

      } else if ( !strcasecmp(type, "LINK") && !strcasecmp(darg, "UDP") ) {

        OpenUdpLink(from, uhost, atoul(snum), port);

      } else {

	ReplyCtcp(from, "ERRMSG Unsupported DCC request");
	Debug("Unsupported DCC request: %s %s", type, darg);

      }
    }
  }
}

/* --- Quoting ---------------------------------------------------- */

#define CTCPQUOTE '\020'
#define CTCPQUOTE_FIRST '\000'
#define CTCPQUOTE_LAST '\037' /* Should be \040 */
#define CTCPQUOTE_OFFSET '\101'

void CtcpQuote(const char *inbuf, char *outbuf)
{
#if 0
  register char c;

  while ( (c=*inbuf++) ) {
    if (c == CTCPQUOTE)
      *outbuf++ = CTCPQUOTE;
    else if ( c >= CTCPQUOTE_FIRST && c <= CTCPQUOTE_LAST ) {
      *outbuf++ = CTCPQUOTE;
      c += CTCPQUOTE_OFFSET;
    }
    *outbuf++ = c;
  }
  *outbuf = NIL;
#else
  strcpy(outbuf, inbuf);
#endif
}

void CtcpUnquote(const char *inbuf, char *outbuf)
{
#if 0
  register char c;

  while ( (c=*inbuf++) ) {
    if (c == CTCPQUOTE) {
      c = *inbuf++;
      if ( c >= (CTCPQUOTE_FIRST + CTCPQUOTE_OFFSET) &&
	   c <= (CTCPQUOTE_LAST + CTCPQUOTE_OFFSET) )
        c -= CTCPQUOTE_OFFSET;
    }
    *outbuf++ = c;
  }
  *outbuf = NIL;
#else
  strcpy(outbuf, inbuf);
#endif
}
