#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>

#include "dancer.h"
#include "function.h"
#include "explain.h"

#define DEF_SIZE 10
#define MAX_LINE 256


struct Explain {
  char *word;
  char *explain;
};

struct Explain *explist;
int numberofexplains = 0;
int expalloced;


/* --- FileExplain ------------------------------------------------ */

bool FileExplain(char *name)
{
  FILE *f;
  char line[MAXLINE];
  char *pnt;

  if (name && *name && (f = fopen(name, "r"))) {
    while (fgets(line, MAXLINE, f)) {
      if ((line[0] != '#') &&
          (line[0] != '\n')) {
        ExplainAddItem(line);
      }
    }
    fclose(f);
    return TRUE;
  }
  return FALSE;
}

/* --- ExplainAdded ----------------------------------------------- */

int comparis(struct Explain *e1, struct Explain *e2)
{
  return strcasecmp((*e1).word, (*e2).word);
}

void ExplainAdded(void)
{
  qsort((void *)explist, numberofexplains,
        sizeof(struct Explain), (int (*)(const void *, const void *))comparis);
}

/* --- InitExplain ------------------------------------------------ */

void InitExplain(char *name)
{
  FILE *f, *fa;
  char line[MAXLINE];

  explist = (struct Explain *)malloc(sizeof(struct Explain) * DEF_SIZE);
  memset(explist, 0, sizeof(struct Explain) * DEF_SIZE);
  expalloced = DEF_SIZE;

  FileExplain(name);
  sprintf(line, "%s.add", name);
  FileExplain(line);

  ExplainAdded();
}

/* --- FindExplain ------------------------------------------------ */

bool FindExplain(char *match)
{
  int n;

  for (n=0; n < numberofexplains; n++)
    if (Match(explist[n].word, match))
      return TRUE;
  return FALSE;
}

/* --- AddExplain ------------------------------------------------- */

bool AddExplain(char *word, char *explain)
{
  if (expalloced == numberofexplains) {
    expalloced += DEF_SIZE;
    explist = (struct Explain *)
      realloc(explist, sizeof(struct Explain) * expalloced);
    if (!explist)
      return FALSE;
  }
  explist[numberofexplains].word = strdup(word);
  explist[numberofexplains].explain = strdup(explain);

  ++numberofexplains;
  return TRUE;
}

/* --- ExplainCleanup --------------------------------------------- */

void ExplainCleanup(void)
{
  int n;

  for (n=0; n < numberofexplains; n++) {
    free(explist[n].word);
    free(explist[n].explain);
  }
  free(explist);
  numberofexplains = 0;
  explist = NULL;
}

/* --- ShowExplain ------------------------------------------------ */

int ShowExplain(char *from, char *match, bool search)
{
#if 1
  int len, n;
  int col = 0;
  int matches = 0;
  int firsthit = 0;
  char buffer[MAX_LINE];
  char *pnt = buffer;

  for (n=0; n < numberofexplains; n++) {
    if (Match((search ? explist[n].explain : explist[n].word), match)) {
      len = strlen(explist[n].word)+1;
      if (++matches == 1) {
        firsthit = n;
      }
      else {
        if ((col + len) > MAX_LINE) {
          Sendf(from, "%s", buffer);
          pnt = buffer;
          col = 0;
        }
      }
      sprintf(pnt, "%s ", explist[n].word);
      col += len;
      pnt += len;
    }
  }
  if (matches == 1)
    Sendf(from, "'%s' %s", explist[firsthit].word, explist[firsthit].explain);
  else if (matches && col)
    Sendf(from, "%s", buffer);
  return matches;

#else

  int n;
  int theonly=-1;
  int matches=0;
  for(n=0; n<numberofexplains; n++) {
    if(Match(explist[n].word, match)) {
      ++matches;
      if(theonly<0)
        theonly=n;
    }
  }
  if(1 == matches) {
    Sendf(from, "'%s' %s", explist[theonly].word, explist[theonly].explain);
  }
  else if(matches) {
    char buffer[MAX_LINE];
    char *pnt=buffer;
    int col=0;
    for(n=theonly; n<numberofexplains; n++) {
      if(Match(explist[n].word, match)) {
        int len = strlen(explist[n].word)+1;

        if(col + len > MAX_LINE) {
          Sendf(from, "%s", buffer);
          pnt=buffer;
          col = 0;
        }
        sprintf(pnt, "%s ", explist[n].word);
        col += len;
        pnt += len;
      }
    }
    if(col)
      Sendf(from, "%s", buffer);
  }
  return matches;
#endif
}

/* --- ExplainAddItem --------------------------------------------- */

int ExplainAddItem(char *line)
{
  char *param, *s;
  char word[MAX_LINE];

  if (!strtok(line, "="))
    return 1;
  param = strtok(NULL, "\n");
  if (!param)
    return 1;

  s = strtok(line, ",");
  do {
    while (isspacechar(*s))
      s++;
    if (FindExplain(s))
      return 2;
    if (!AddExplain(s, param))
      return 1;

  } while (s = strtok(NULL, ","));
  return 0;
}

/* --- ExplainSave ------------------------------------------------ */

int ExplainSave(char *filename)
{
  char tempname[MIDBUFFER];
  FILE *out;
  int ok = TRUE;
  int n;
  sprintf(tempname, "%s~", filename);

  out = fopen(tempname, "w");
  if (out) {
    if (0 <= fprintf(out, "# Dancer explain list version: " VERSIONMSG "\n")) {
      for (n=0; n < numberofexplains; n++) {
        if (0 > fprintf(out, " %s=%s\n", explist[n].word, explist[n].explain)) {
          ok = FALSE;
          break;
        }
      }
    }
    else
      ok = FALSE;
    fclose(out);
    if (ok) {
      rename(tempname, filename);
    }
  }
  return ok;
}

/* --- ExplainDel ------------------------------------------------- */

bool ExplainDel(char *word)
{
  int n;
  bool removed = FALSE;

  for (n=0; n < numberofexplains; n++) {
    if (!strcasecmp(explist[n].word, word)) {
      free(explist[n].word);
      free(explist[n].explain);
      numberofexplains--;
      for(; n < numberofexplains; n++) {
        explist[n].word = explist[n+1].word;
        explist[n].explain = explist[n+1].explain;
      }
      removed= TRUE;
      break;
    }
  }
  return removed;
}

/* ---------------------------------------------------------------- */

/*
 * Explaination keywords. Works a bit like 'man -k'
 * Certain explainations can be logically grouped.
 *
 * Example:
 *
 *   'ircII' and 'mIRC' may both belong to the group 'client', and
 *   'ircII' may also belong to the group 'unix'.
 *
 *   "EXPLAIN -k client" will result in "ircII, mIRC"
 *   "EXPLAIN -k unix" in "ircII, linux, solaris, sunos"
 *
 * FindExplainKey() first searches for keywords, if none were found
 * it searches the usual words.
 *
 * Represented as 'word key1 key2=description'. Internal representation
 * is word = 'word key key'. Extract 'word' with sscanf("[^= \n]) on
 * normal searches.
 */
