#ifdef RCS
static char rcsid[]="$Id: flood.c,v 2.4 1998/03/23 18:31:09 dast Exp $";
#endif
/******************************************************************************
 *                    Internetting Cooperating Programmers
 * ----------------------------------------------------------------------------
 *
 *  ____    PROJECT
 * |  _ \  __ _ _ __   ___ ___ _ __ 
 * | | | |/ _` | '_ \ / __/ _ \ '__|
 * | |_| | (_| | | | | (_|  __/ |   
 * |____/ \__,_|_| |_|\___\___|_|   the IRC bot
 *
 * Main authors:
 *  - Bjrn Reese <breese@imada.ou.dk>
 *  - Daniel Stenberg <Daniel.Stenberg@sth.frontec.se>
 *
 * Contributing authors (and ports):
 *  - Philippe Brand <phb@telesys-innov.fr>
 *  - Bjrn Stenberg <Bjorn.Stenberg@sth.frontec.se>
 *
 * All files in this archive are subject to the GNU General Public License.
 * 
 *               HTTP://www.imada.ou.dk/~breese/dancer.html
 *                FTP://ftp.netcom.com/pub/ra/raydar/Dancer
 *
 * $Source: /home/hardcore/dast/dancer/src/RCS/flood.c,v $
 * $Revision: 2.4 $
 * $Date: 1998/03/23 18:31:09 $
 * $Author: dast $
 * $State: Exp $
 * $Locker:  $
 *
 * ---------------------------------------------------------------------------
 * $Log: flood.c,v $
 * Revision 2.4  1998/03/23 18:31:09  dast
 * ACTION no longer causes CTCP FLOOD kick
 *
 * Revision 2.3  1997/11/05 22:00:25  dast
 * breese fixed
 *
 * Revision 2.2  1997/07/15 07:45:42  dast
 * Fixed the alert mode to increase alert level when it should
 *
 * Revision 2.1  1997/05/28 06:21:56  dast
 * Colour kick patch applied, patch done by Bobby
 *
 * Revision 1.11  1997/04/28 16:39:09  dast
 * translations
 *
 * Revision 1.10  1997/01/05 21:11:47  dast
 * made multiple joins check kick all users if triggered
 * made the limit SETable
 *
 * Revision 1.9  1996/12/12 19:27:10  dast
 * changes two comparisons to work with the exact throeshold entered
 *
 * Revision 1.8  1996/12/09 18:24:55  dast
 * extern levels
 *
 * Revision 1.7  1996/11/04 19:16:34  dast
 * breese fixes
 *
 * Revision 1.6  1996/09/03 15:23:50  dast
 * Now prevents logging of alert mode level when above 1000
 *
 * Revision 1.5  1996/08/30 11:29:13  dast
 * uses the 'now' variable
 *
 * Revision 1.4  1996/08/20 14:33:36  dast
 * new alertmode support
 *
 * Revision 1.3  1996/08/06 10:52:30  dast
 * added AlertMode
 *
 * Revision 1.2  1996/07/26 11:34:58  dast
 * breese clean up
 *
 * Revision 1.1  1996/07/18 13:51:50  dast
 * Initial revision
 *
 *
 *****************************************************************************/

#include <string.h>
#include <ctype.h>
#include "dancer.h"
#include "transfer.h"
#include "user.h"
#include "flood.h"
#include "bans.h"

extern time_t now;

extern bool uppercheck; /* check users for uppercase violations! */
extern bool beepcheck;  /* check users for beep violations! */
extern bool colourcheck;  /* check users for colourcode violations! */
extern bool ctcpmode;  /* CTCP flood checks */
extern bool mute;
extern itemguest *guestHead;
extern long levels[];

int floodrate, floodtime;  /* 'floodrate' messages within 'floodtime' secs */
int floodrepeatrate, floodrepeattime;  /* 'floodrepeatrate' repetitions within 'floodrepeattime' secs */
int floodbeeps;
int floodjoins; /* max amount of joins from the same hostpattern */

/****************************************************************************
*     _    _           _   __  __           _      
*    / \  | | ___ _ __| |_|  \/  | ___   __| | ___ 
*   / _ \ | |/ _ \ '__| __| |\/| |/ _ \ / _` |/ _ \
*  / ___ \| |  __/ |  | |_| |  | | (_) | (_| |  __/
* /_/   \_\_|\___|_|   \__|_|  |_|\___/ \__,_|\___|
*
***************************************************************************/

#define ALERT_TIMEOUT (10*60) /* at least 10 minutes from the last
                                 alert ON */
int AlertMode(Alert what)
{
  static time_t alerted=0;
  static int level=0;
  switch(what) {
  case ALERT_ON:
    alerted = now;
    level++;
    break;
  case ALERT_RED:
    level+=100;
    alerted = now;
    break;
  case ALERT_OFF:
    if(alerted && (alerted+ALERT_TIMEOUT<now) ) {
      Log(LOG, "Alert Mode OFF");
      alerted = 0;
      level = 0;
    }
    return 0;
  default:
    break;
  }
  if(level > 0 && level < 100)
    Logf(LOG, "Alert Mode level %d", level);
  else if(level >= 100 && level < 1000)
    Logf(LOG, "Alert Mode *RED* level %d", level);
  return level;
}

/* --- Checks ----------------------------------------------------- */

bool Warning(itemguest *p, char *msg, char *kmsg)
{
  int level;

  level = p->ident->user ? p->ident->user->level : 0;
  if ( level < FLOODMIDHIGH ) {
    p->warnings++;
    if ( level < FLOODLOWMID ) {
      /* low -> mid */
      if ( p->warnings > WARNLOW ) {
        StickyKick(p, (p->warnings > 1 ? GetDefaultText(msg_no_more_mr_nice_bot) : kmsg));
        p->warnings = 0;
        return TRUE; /* Kicked */
      }
    } else if ( p->warnings > WARNMID ) {
      /* mid -> high */
      StickyKick(p, (p->warnings > 1 ? GetDefaultText(msg_no_more_mr_nice_bot) : kmsg));
      p->warnings = 0;
      return TRUE; /* Kicked */
    }
    if (!mute)
      Actionf(GetDefaultText(msg_usually_kicks_X), msg, p->warnings, p->ident->nick);
  } /* high -> inf */
  return FALSE;
}

void FloodCheck(itemguest *p)
{
  if (now <= (p->posttime + floodtime)) {
    if (++p->postsame >= floodrate)
        Warning(p, GetDefaultText(msg_flooders), GetDefaultText(msg_no_flooding));
  } else
    p->postsame = 0;
}

void Check(itemguest *p, char *line)
{
  long length;
  long uppers=0;
  long beeps=0, colours=0;

  if (!line) return;

  for (length=0; line[length]; ++length) {
    if (uppercheck && isupper(line[length]))
      uppers++;
    if (beepcheck && line[length]=='\x07')
      beeps++;

    /* Colourcode kick      - RiffRaff!RiffRaff@login.dknet.dk */
    if( colourcheck && line[length] == '\x03'	&& isdigit(line[length+1]) ){
      colours++;
    }
  }
  if (beeps ) {
    if(floodbeeps >= beeps)
      Warning(p, GetDefaultText(msg_beepers), GetDefaultText(msg_kickmsg_no_beepers));
    else
      StickyKick(p, GetDefaultText(msg_kickmsg_no_beepers));
  }
  if (length > 10) {
    if (uppers*1000/length > 750) {
      /* This is a serious uppercase violation */
      Warning(p, GetDefaultText(msg_shouters), GetDefaultText(msg_no_shouting));
    }
  }
  if (colours) {
    Warning(p, GetDefaultText(msg_colour_users), GetDefaultText(msg_kickmsg_no_colours));
  }
}

void RepeatCheck(itemguest *p, char *line)
{
  unsigned long hash=0;

  if (!line) return;
  while(*line)
    hash = (hash<<1) + *line++;

  if ((hash == p->hashpost) && ((p->hashtime + floodrepeattime) >= now)) {
    if (++p->hashsame >= floodrepeatrate)
      Warning(p, GetDefaultText(msg_repeaters), GetDefaultText(msg_no_repeaters));
  } else {
    p->hashpost = hash;
    p->hashtime = now;
    p->hashsame = 1;
  }
}

/* --- Flood checks ----------------------------------------------- */

void AvalanceCheck(itemguest *w, char *param)
{
  int cntone = 0, cntesc = 0, cntbeep = 0;
  char c;

  if (!param) return;

  while ((c=*param++)) {
    switch (c) {
    case '\001':
      cntone++;
      break;
    case '\x1b':
      if (*param == '\026')
	cntesc++;
	break;
    case '\007':
      cntbeep++;
      break;
    default:
      break;
    }
  }
  if ( cntone > 5 ) { /* Kick without warning */
    StickyKick(w, GetDefaultText(msg_ctcp_bomb_detected));
    return;
  } else if ( cntesc > 4 ) {
    StickyKick(w, GetDefaultText(msg_tsunami_detected));
    return;
  } else if ( cntbeep > 5 ) {
    StickyKick(w, GetDefaultText(msg_beep_flood_detected));
    return;
  }
}

bool CTCPFloodCheck(itemguest *w, bool action)
{
/*
 * Number of codes
 */
#define MAX_ACTION 6
#define MAX_CTCP   3

/*
 * Shortest time interval the MAX number of codes are accepted to flood.
 */
#define TIME_ACTION 4
#define TIME_CTCP   6

  int compare=999;
  bool flooding = FALSE;
  static time_t ntime[MAX_CTCP];
  static time_t atime[MAX_ACTION];
  static int nctcp;
  static int actcp;
  int tmp;

  if(action) {
    tmp = actcp % MAX_ACTION;
    atime[tmp] = now;
    if(actcp<MAX_ACTION-1)
      tmp = -1;
    actcp++;
  }
  else {
    tmp = nctcp % MAX_CTCP;
    ntime[tmp] = now;
    if(nctcp<MAX_CTCP-1)
      tmp = -1;
    nctcp++;
  }

  if(tmp>=0) {
    /*
     * Compare to fetch a possible hit.
     */
    if(action) {
#if 0
      int n;
      for(n=0; n<MAX_ACTION; n++) {
        fprintf(stderr, "(%d) %d ", n, now-atime[n]);
      }
      fprintf(stderr, "\nnow %d <-> %d\n",
              tmp, (tmp+1)%MAX_ACTION);
#endif
      compare = now-atime[(tmp+1)%MAX_ACTION];
    }
    else {
      compare = now-ntime[(tmp+1)%MAX_CTCP];
    }
  }
  /*
   * Now we have a time to compare with.
   */
    
  if (w) {
    if(action) {
      if(compare<=TIME_ACTION)
        flooding=TRUE;
    }
    else {
      if(compare<=TIME_CTCP)
        flooding=TRUE;
      }
    
    if (TRUE == flooding) {
#if 0
      sprintf(buf, "Flood: CTCP%s flood by %s!%s",
              action?" ACTION":"", w->ident->nick,
              w->ident->host);
      Multicast(REPORTCAST, buf);
#endif
      if(ctcpmode && !action) {
        /* don't do this on ACTION floods, but we can still report ACTION
           floods to trigger ignore */
        Warning(w, "CTCP flooders", "CTCP flood detected");
      }
    }
  } else /* a ctcp request from outside the channel */
    flooding = TRUE; /* Not really flooding, but we ignore it */
  return flooding;
}

/* ---------------------------------------------------------------- */

/*
 * This should kick users that try to join using the same user account
 * for the Xth time.
 */

void MultiCheck(char *domain)
{
  itemguest *p;
  int count = 0; /* none so far! */

  if (domain) {
    for (p = Succ(guestHead); p; p = Succ(p)) {
      if (p->ident->userdomain &&
          !strcasecmp(p->ident->userdomain, domain)) {
        ++count; /* count this */
      }
    }
    if (count > floodjoins) {
      AlertMode(ALERT_ON);

      for (p = Succ(guestHead); p; p = Succ(p)) {
	if (p->ident->userdomain &&
	    !strcasecmp(p->ident->userdomain, domain)) {
	  StickyKick(p, "*bang* too many joined users");
	}
      }
    }
  }
}

