#ifdef RCS
static char rcsid[]="$Id: news.c,v 1.3 1997/11/24 23:35:51 dast Exp $";
#endif
/******************************************************************************
 *                    Internetting Cooperating Programmers
 * ----------------------------------------------------------------------------
 *
 *  ____    PROJECT
 * |  _ \  __ _ _ __   ___ ___ _ __ 
 * | | | |/ _` | '_ \ / __/ _ \ '__|
 * | |_| | (_| | | | | (_|  __/ |   
 * |____/ \__,_|_| |_|\___\___|_|   the IRC bot
 *
 * Main authors:
 *  - Bjrn Reese <breese@imada.ou.dk>
 *  - Daniel Stenberg <Daniel.Stenberg@sth.frontec.se>
 *
 * Contributing authors (and ports):
 *  - Philippe Brand <phb@telesys-innov.fr>
 *  - Bjrn Stenberg <Bjorn.Stenberg@sth.frontec.se>
 *  - Andrew Gray <DarkDesigns@trader.com>
 *
 * All files in this archive are subject to the GNU General Public License.
 * 
 *               HTTP://www.imada.ou.dk/~breese/dancer.html
 *                FTP://ftp.netcom.com/pub/ra/raydar/Dancer
 *
 * $Source: /home/hardcore/dast/dancer/src/RCS/news.c,v $
 * $Revision: 1.3 $
 * $Date: 1997/11/24 23:35:51 $
 * $Author: dast $
 * $State: Exp $
 * $Locker:  $
 *
 * ---------------------------------------------------------------------------
 * $Log: news.c,v $
 * Revision 1.3  1997/11/24 23:35:51  dast
 * lame cleanup crash removed
 *
 * Revision 1.2  1997/08/25 17:51:29  dast
 * check for list before destroying it
 *
 * Revision 1.1  1997/05/28 06:23:21  dast
 * Initial revision
 *
 *****************************************************************************/

/**************************************
                  Suggested way of a NEWS system for Dancer
                  =========================================

 The NEWS system is a kind of mass-tell that shows information to persons with
a certain level or higher when they join. Each NEWS item is only showed once
to each user (if not otherwise explicitly requested). The NEWS items will
automatically be removed after a certain number of days in the list.
 The system will of course prevent too many NEWS items to get displayed if a
user joins and X news items are to get displayed where X is larger than a
certain limit. The system will then fall back to say that "there are X news
items to get displayed, use NEWSREAD to read them".


Suggested commands:

NEWSADD [ -e expires] [-U] [-p match] <level> <text>
  Adds a new NEWS item to the list.

  -U marks the news item as... urgent. ;)

  <match> is a wildcard match the receiver must match to receive the news.

  <expires> is the number of days the news is valid. Default will be set in
   the .config file controlling the bot (internal default 90 days).

  <level> is the least level required for a user to get this info displayed.
   Level 0 (or 'all') makes all registered users get the info.

  When invoked, this command will output the 'newsid' which can be used to
  remove it or explicitly read it with NEWSREAD.

NEWSDEL <id>
  Removes the specified NEWS item from the list.

NEWSLIST [number of items]
  Lists the X latest NEWS items from the list. Detailed with author, date, id
  and level info.

NEWSREAD [id] (actually a TELLME alias)
  With no argument, this presents the oldest unread NEWS entry. With argument
  it presents the item associated with the specified id.
  -a reads all items at once
  -c clear all, make all items marked as read without showing them

User-data:

AUTONEWS
  Controls how a user gets the news presented on join. Three different options:
  'NEVER' - I don't want to see any news at all (at join)
  'INFO' - I just want to be told there is news (and number of unread items)
  '<number>' - I want to see the first (oldest) news item on joining, when the
               news level is higher than this number (default is 0). If none
               match but there are unmatched ones, the INFO way will be used

Info stored per NEWS item:

 <id>     - 32 bit number (unique number)
 <date>   - 32 bit number (time of entry, seconds since 1970)
 <level>  - 32 bit number (lowest level required to get this)
 <author> - string (who wrote this)
 <flags>  - string (URGENT?)
 <expire> - 32 bit number (days to keep the entry)
 <match>  - string (userhost match pattern)
 <text>   - text to end of line

NEW ONES
 <count>  - read counter
 <last>   - time of last reading


***************************************/

#include <string.h>

#include "dancer.h"
#include "list.h"
#include "user.h"
#include "news.h"

itemnews *newsHead;

static long newsid; /* use the function to access */

long newscounter=0;

extern char newsfile[];
extern time_t now;
extern itemident *current;
extern long levels[];

/* ------------------------------------------------------- */

void NewsFree(void *v)
{
  itemnews *n;
  if(n=(itemnews *)v) {
    if(n->author)
      free(n->author);
    if(n->news)
      free(n->news);
    if(n->match)
      free(n->match);

    newscounter--;
  }
}

/* returns TRUE on error */
bool NewsAddItem(char *author,
                 long level,
                 long id,
                 time_t time,
                 long flags,
                 long expire,
                 char *match,
                 char *news)
{
  itemnews *n;
  n = (itemnews *)NewEntry(sizeof(itemnews));
  if(n) {
    AddTail(newsHead, n);
    n->level  = level;
    n->id     = id;
    n->time   = time;
    n->flags  = flags;
    n->expire = expire;
    n->author = strdup(author);
    n->match  = strdup(match?match:"*");
    n->news   = strdup(news);

    newscounter++;
  }
  else
    return TRUE;
  return FALSE;
}

void NewsSave(void)
{
  FILE *f;
  itemnews *n;
  char tempfile[MIDBUFFER];
  bool ok=TRUE;
  
  if(!newsHead || !*newsfile) /* no news list or no file */
    return;
  
  sprintf(tempfile, "%s~", newsfile);
  f = fopen(tempfile, "w");
  if(f) {
    fprintf(f, "News seqno: %d\n", newsid);
    for(n=Succ(newsHead); n; n=Succ(n)) {
      while((n->time + (n->expire * 24*3600)) < now) {
        n = DeleteEntry(newsHead, n, NewsFree);
        if(!n)
          break;
      }
      if(!n)
        break;
      
      if(0>fprintf(f, "%s %d %d %d %d %d %s :%s\n",
                   n->author,
                   n->level,
                   n->id,
                   n->time,
                   n->flags,
                   n->expire,
                   n->match,
                   n->news)) {
        ok = FALSE;
        break;
      }
    }
    fclose(f);
    if(ok)
      rename(tempfile, newsfile);
  }
}

void NewsInit(void)
{
  FILE *f;
  char buf[MAXLINE];
  char news[BIGBUFFER];
  char author[BIGBUFFER];
  char match[MIDBUFFER];
  long flags;
  long expire;
  long id;
  time_t time;
  long level;
  newsHead = CreateList(sizeof(itemnews));

  if(*newsfile && (f=fopen(newsfile, "r")) ) {
    while (fgets(buf, MAXLINE, f)) {
      news[0]=0;
      if(1 == sscanf(buf, "News seqno: %d", &newsid))
        ;
      else if (8 <= sscanf(buf, "%"BIGBUFFERTXT"s %d %d %d %d %d "
                           "%"MIDBUFFERTXT"s :%"BIGBUFFERTXT"[^\n]",
                           author, &level, &id, &time, &flags, &expire,
                           match, news)) {
        NewsAddItem(author, level, id, time, flags, expire, match, news);
      }
    }
    fclose(f);
  }
}

void NewsCleanup(void)
{
  NewsSave();
  if(newsHead)
    DestroyList((void **)newsHead, NewsFree);
}

/* returns TRUE if something went wrong */
bool NewsDelete(long id, long flags)
{
  itemnews *n;
  if(!newsHead)
    return TRUE;
  for(n=Succ(newsHead); n; n=Succ(n)) {
    if(n->id == id) {
      n = DeleteEntry(newsHead, n, NewsFree);
      return FALSE;
    }
  }
  return TRUE;
}

/***********************************************************************
 *
 * NewsRead()
 * ==========
 * Returns the number of unread news including the pointed one.
 *
 * The input ID is the high water mark to use.
 *
 * The newsp pointer points to a 'itemnews' pointer that will point out
 * the current news item to read.
 *
 ***********************************************************************/

long NewsRead(long id, itemnews **newsp)
{
  itemnews *n;
  long unread=0;
  if(newsp)
    *newsp = NULL; /* default to nothing */
  if(!newsHead)
    return unread;
  for(n=Succ(newsHead); n; n=Succ(n)) {
    if(n->id > id && current->level>=n->level) {
      if(Match(current->host, n->match)) {
        if(newsp && !*newsp)
          *newsp = n;
        unread++;
      }
    }
  }
  return unread;
}

long NewsID(void)
{
  return ++newsid;
}

/***********************************************************************
 *
 * NewsReport()
 *
 * Report news for a user at join.
 *
 **********************************************************************/
void NewsReport(itemident *ident)
{
  itemnews *n;
  if(NewsRead(ident->user->newsid, &n)) {
    Sendf(ident->nick,
          GetText(msg_news_report),
          n->news,
          n->author,
          TimeAgo(n->time));
    ident->user->newsid = n->id;
  }
}

static long counternews(void)
{
  itemnews *n;
  long count=0;
  for(n=Succ(newsHead); n; n=Succ(n)) {
    if(current->level<n->level ||
       (current->level<=LEVELCHANOP &&
        !Match(current->host, n->match)))
      continue;
    count++;
  }
  return count;
}

void NewsList(char *from, long lines)
{
  itemnews *n;
  int count=0;
  int max = counternews();
  for(n=Succ(newsHead); n; n=Succ(n)) {
    if(current->level<n->level ||
       (current->level<=LEVELCHANOP &&
        !Match(current->host, n->match)))
      continue;
    if(count+lines >= max) {
      Sendf(from, GetText(msg_news_list),
            n->id>current->user->newsid ? GetText(msg_unread) : "",
            n->id,
            n->level,
            n->match,
            n->author,
            TimeAgo(n->time));
    }
    count++;
  }
  if(!count)
    Sendf(from, GetText(msg_no_news));
}
