#ifdef RCS
static char rcsid[]="$Id: seen.c,v 1.23 1998/04/14 11:28:33 dast Exp dast $";
#endif
/******************************************************************************
 *                    Internetting Cooperating Programmers
 * ----------------------------------------------------------------------------
 *
 *  ____    PROJECT
 * |  _ \  __ _ _ __   ___ ___ _ __ 
 * | | | |/ _` | '_ \ / __/ _ \ '__|
 * | |_| | (_| | | | | (_|  __/ |   
 * |____/ \__,_|_| |_|\___\___|_|   the IRC bot
 *
 * Main authors:
 *  - Bjrn Reese <breese@imada.ou.dk>
 *  - Daniel Stenberg <Daniel.Stenberg@sth.frontec.se>
 *
 * Contributing authors (and ports):
 *  - Philippe Brand <phb@telesys-innov.fr>
 *  - Bjrn Stenberg <Bjorn.Stenberg@sth.frontec.se>
 *  - Andrew Gray <DarkDesigns@trader.com>
 *
 * All files in this archive are subject to the GNU General Public License.
 * 
 *               HTTP://www.imada.ou.dk/~breese/dancer.html
 *                FTP://ftp.netcom.com/pub/ra/raydar/Dancer
 *
 * $Source: /home/hardcore/dast/dancer/src/RCS/seen.c,v $
 * $Revision: 1.23 $
 * $Date: 1998/04/14 11:28:33 $
 * $Author: dast $
 * $State: Exp $
 * $Locker: dast $
 *
 * ---------------------------------------------------------------------------
 * $Log: seen.c,v $
 * Revision 1.23  1998/04/14 11:28:33  dast
 * Translated a missed seen reason
 *
 * Revision 1.22  1997/07/15 07:46:48  dast
 * Error-check one of those strdup()s only a million of them to go...
 *
 * Revision 1.21  1997/04/28 16:43:06  dast
 * translations added
 * added sanity checks in the seenadd routines
 *
 * Revision 1.20  1997/01/05 21:15:21  dast
 * dynamic seen counter
 * seensaves are prevented if no change has been done
 *
 * Revision 1.19  1996/12/12 18:59:05  dast
 * corrrected the header
 *
 * Revision 1.18  1996/12/11 18:35:55  dast
 * frees the ->host in the cleanup too
 *
 * Revision 1.17  1996/12/06 15:56:05  dast
 * safer save
 * timestamped nicks
 * improved seen-recent
 *
 * Revision 1.16  1996/12/03 17:12:09  dast
 * slightly changed seen
 *
 * Revision 1.15  1996/12/03 17:01:59  dast
 * minor detailed that -Wall complained at
 *
 * Revision 1.14  1996/11/04 19:18:10  dast
 * breese fixed a seen bug
 *
 * Revision 1.13  1996/08/30 11:29:13  dast
 * uses 'now'
 *
 * Revision 1.12  1996/08/21 11:17:31  dast
 * breese's update
 *
 * Revision 1.11  1996/08/05 13:34:13  dast
 * nothing really ;)
 *
 * Revision 1.8  1996/07/25 08:35:00  dast
 * changed the cp usage to fcopy()
 *
 * Revision 1.7  1996/07/10 16:15:00  dast
 * Added the source header.
 *
 *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#include "dancer.h"
#include "function.h"
#include "user.h"
#include "transfer.h"
#include "seen.h"


/* --- Global ----------------------------------------------------- */

extern time_t now;

extern char seenfile[];
extern itemguest *guestHead;
extern itemopt option;
extern bool chat, seenbackup;
extern int seenmonths;

int numSeenHosts, numSeenNicks;

struct Seenstruct SeenHead = {
  NULL, NULL, NULL,
  NULL, NULL,
  0, 0, 0,
  NULL, NULL
};

itemseen *seenHead = &SeenHead;

itemaux **hashnick  = NULL;
itemseen **hashsite = NULL;

static long changedseenitem=0;

/* --- SeenReport ------------------------------------------------- */

void SeenReport(char *from, char *who, itemseen *ps)
{
  switch (ps->type) {
  case SEENLEAVE:
    Sendf(from, GetText(msg_seenleave),
          who, ps->usite, TimeAgo(ps->departure));
    break;
  case SEENQUIT:
    Sendf(from, GetText(msg_seenquit),
          who, ps->usite, TimeAgo(ps->departure), ps->msg ? ps->msg : "");
    break;
  case SEENKICK:
    Sendf(from, GetText(msg_seenkick), who, ps->usite,
          TimeAgo(ps->departure), ps->by, ps->msg ? ps->msg : "");
    break;
  case SEENLEFT:
    Sendf(from, GetText(msg_seenleft),
          who, ps->usite, TimeAgo(ps->departure));
    break;
  case SEENQUITED:
  case SEENERROR:
    Sendf(from, GetText(msg_seenquited),
          who, ps->usite, TimeAgo(ps->departure));
    break;
  case SEENKICKED:
    Sendf(from, GetText(msg_seenkicked),
          who, ps->usite, TimeAgo(ps->departure), ps->by);
    break;
  case SEENCHANGE:
    Sendf(from, GetText(msg_seenchange),
          who, ps->usite, TimeAgo(ps->departure));
  default:
    break;
  }
}

/* --- SeenMostNick ----------------------------------------------- */

itemaux *SeenMostNick(char *nick)
{
  itemaux *p, *pp, *m = NULL;
  long max = 0;

  if ( pp=hashnick[HashU(nick)] ) { /* beginning of list */
    for (p = pp; p; p = p->next) {
      if ( !strcasecmp(p->nick, nick) ) {
        if ( p->count >= max ) {
          max = p->count;
          m = p;
        }
      }
    }
  }
  return m;
}

itemaux *SeenRecentNick(char *nick)
{
  itemaux *p, *pp, *m = NULL;
  time_t recent = 0;

  if ( pp=hashnick[HashU(nick)] ) {
    for (p = pp; p; p = p->next) {
      if ( !strcasecmp(p->nick, nick) ) {
        if (p->root->departure > recent) {
          recent = p->root->departure;
          m = p;
        }
      }
    }
  }
  return (m);
}


/* --- SeenFindNick ----------------------------------------------- */

void SeenFindNick(char *from, char *nick, char cflag)
{
  itemaux *p, *pp;
  long max = 0;
  time_t recent = 0;
  bool hit = FALSE;

  if ( pp=hashnick[HashU(nick)] ) { /* beginning of list */

    for (p = pp; p; p = p->next) {

      if ( !strcasecmp(p->nick, nick) ) {

        switch (cflag) {
        case 'A': /* All */
	  hit = TRUE;
          SeenReport(from, nick, p->root);
          break;
        case 'M': /* Most often used */
          hit = TRUE;
          if ( p->count > max ) {
            max = p->count;
            pp = p;
          }
          break;
        default:  /* Most recent */
          hit = TRUE;
	  /*
	   * Bagder September 22nd, 1998:
	   * Here, we check if we have the timestamp on the nick if there is
	   * one. If we do, we use that to check for the most
	   * recent nick instead of checking the host's timestamp. Should
	   * give us much more accurate information. Only if there is no set
	   * nick departure, we use the root departure time! 
	   */
	  if( p->departure ) {
	    if( p->departure > recent ) {
	      recent = p->departure;
	      pp = p;
	    }
	  }
          else if ( p->root->departure > recent ) {
            recent = p->root->departure;
            pp = p;
          }
          break;
        }
      }
    }
  }

  if (hit) {
    if (pp)
      SeenReport(from, nick, pp->root);
  } else
    Sendf(from, GetText(msg_never_seen), nick);
}

/* --- SeenNick --------------------------------------------------- */

void SeenNick(char *from, char *who, char option)
{
  itemaux *pa;
  itemguest *p;

  if ( ( (p=FindNick(who)) || (p=FindSplitNick(who)) ) &&
      ('A' != toupper(option))  ) {
    if(p->flags.split)
      Sendf(from, GetText(msg_seensplit), who);
    else
      Send(from, GetText(msg_seen_already_here));
           /*GIMMEPICK(MSG_SEEN_ALREADY_HERE_1, 3));*/
    return;
  }

  SeenFindNick(from, who, option);
}

/* --- SeenFindHost ----------------------------------------------- */

itemseen *SeenFindHost(itemseen *p, char *who, ulong sig)
{
  while (p) {
    /* Pre-check by signatures to speed up the comparison */
    if ( ((p->signature & sig) == sig) && Match(p->host, who) )
      return (p);
    p = p->link;
  }
  return NULL;
}

/* --- SeenHost --------------------------------------------------- */

#define SEEN_MAX_DISPLAYED 15

void SeenHost(char *from, char *who)
{
  itemseen *p, *pp;
  itemaux *pa, *pam;
  ulong sig, max;
  int cnt = 0;
 
  sig = HashSignature(who);
  for (p = seenHead->link; pp=SeenFindHost(p, who, sig); p = pp->link) {
    max = 0;
    for (pa = pam = pp->first; pa; pa = pa->link) {
      if (pa->count > max) {
        pam = pa;
        max = pa->count;
      }
    }
    SeenReport(from, pam->nick, pp);
    if(++cnt>=SEEN_MAX_DISPLAYED) {
      Send(from, GetText(msg_seen_bad_pattern));
      break;
    }
  }
  if (cnt == 0)
    Sendf(from, GetText(msg_never_seen_match), who);
}

/* --- SeenInsert ------------------------------------------------- */

itemaux *SeenNewAux(itemseen *ps, char *nick)
{
  itemaux *pa;
  unsigned int x;

  if ( pa=(itemaux *)calloc(1, sizeof(struct Auxstruct)) ) {
    x = HashU(nick);
    pa->next      = hashnick[x];
    hashnick[x]   = pa;
    pa->root      = ps;
    pa->link      = ps->first;
    ps->first     = pa;
    pa->nick      = strdup(nick);
    pa->count     = 1;
    pa->departure = ps->departure = now;
  }
  return (pa);
}

void SeenInsert(itemguest *pc, int type, char *by, char *msg)
{
  itemseen *ps;
  itemaux *pa;
  unsigned int x;

  if ( !pc) return; /* Quick fix */

  if ( msg && (*msg == ':') ) msg++;

  changedseenitem++;

  x = Hash(pc->ident->userdomain);
  for (ps = hashsite[x]; ps; ps = ps->next)
    if ( !strcasecmp(pc->ident->userdomain, ps->usite) ) break;

  if (ps) { /* Found existing usite */

    x = HashU(pc->ident->nick);
    for (pa = hashnick[x]; pa; pa = pa->next) {
      if (pa->root == ps) {
        if ( !strcasecmp(pc->ident->nick, pa->nick) )
          break;
      }
    }

    if (pa) {  /* Found nick */

      ps = pa->root;

      pa->count++;

      pa->departure = ps->departure = now;
      ps->type      = type;
      if (ps->by) free(ps->by);
      ps->by        = (by ? strdup(by) : NULL);
      if (ps->msg) free(ps->msg);
      ps->msg       = (msg ? strdup(msg) : NULL);

    } else {  /* Insert new */

      SeenNewAux(ps, pc->ident->nick);
      numSeenNicks++;
    }

  } else {  /* Insert new usite */

    if ( ps=(itemseen *)calloc(1, sizeof(struct Seenstruct)) ) {

      if ( pa=SeenNewAux(ps, pc->ident->nick) ) {

        ps->link       = seenHead->link;
        seenHead->link = ps;
        ps->next       = hashsite[x];
        hashsite[x]    = ps;
        ps->usite      = strdup(pc->ident->userdomain);
        ps->host       = strdup(pc->ident->host);
        ps->signature  = HashSignature(ps->host);
        ps->departure  = now;
        ps->type       = type;
        ps->by         = (by ? strdup(by) : NULL);
        ps->msg        = (msg ? strdup(msg) : NULL);
	numSeenHosts++;

      } else free (ps);
    }

  }
}

/* --- SeenInsertAll ---------------------------------------------- */

void SeenInsertAll(int type, char *by, char *msg)
{
  itemguest *p;

  for(p = Succ(guestHead); p; p = Succ(p))
    SeenInsert(p, type, by, msg);
}

/* --- SeenAdd ---------------------------------------------------- */

itemseen *SeenAdd(char *line)
{
  int x;
  itemseen *p;
  char ahost[MIDBUFFER];
  char aby[MIDBUFFER];
  char amsg[BIGBUFFER];
  int atype;
  ulong adep;

  amsg[0]=0;

  if (3 <= sscanf(line, "%"MIDBUFFERTXT"s %lu %d %"MIDBUFFERTXT"s %*1[:]%"BIGBUFFERTXT"[^\n]", ahost, &adep, &atype, aby, amsg) ) {

    if(NULL == strchr(ahost, '@'))
      /* simple sanity check, a host name must include the @-letter */
      return NULL;

    if ( p=(itemseen *)calloc(1, sizeof(struct Seenstruct)) ) {

      p->link        = seenHead->link;
      seenHead->link = p;
      p->host        = strdup(ahost);
      p->departure   = adep;
      p->type        = atype;
      p->usite       = Userdomain(p->host);
      x              = Hash(p->usite);
      p->next        = hashsite[x];
      hashsite[x]    = p;
      p->signature   = HashSignature(p->host);

      p->by          = (*aby && *aby^'.') ? strdup(aby) : NULL;
      p->msg         = *amsg ? strdup(amsg) : NULL;
      numSeenHosts++;
    }
  }
  return (p);
}

void SeenAddNick(itemseen *ps, char *line)
{
  itemaux *p;
  unsigned int x;
  char nik[NICKLEN+1];
  char buffer[NICKLEN+1];
  ulong cnt;
  time_t departed=0;

  if (ps) {

    switch (sscanf(line, "%"NICKLENTXT"s %d %d", nik, &cnt, &departed)) {
    case 1:
      cnt = 1;
      break;
    case 2:
    case 3:
      break;
    case 0:
    default:
      return;
    }

    /* only add nicknames that use legal characters, as other names are
       likely to be a proof of a broken seen file or similar */
    if(sscanf(nik, "%[0-9A-~-]", buffer) &&
       strcmp(buffer, nik))
      /* not a valid nick! */
      return;

    if ( p=(itemaux *)calloc(1, sizeof(struct Auxstruct)) ) {

      if (NULL == (p->nick = strdup(nik))) {
	free(p);
	return;
      }
      p->count    = cnt;
      p->departure = departed; /* might be 0 if field wasn't set */
      x           = HashU(p->nick);
      p->next     = hashnick[x];
      hashnick[x] = p;
      p->link     = ps->first;
      p->root     = ps;
      ps->first   = p;
      numSeenNicks++;
    }
  }
}

/* --- SeenInit --------------------------------------------------- */

bool SeenInit(void)
{
  FILE *f;
  char line[MAXLINE];
  int ret = FALSE;
  itemseen *p = NULL;

  if (seenbackup) {
    /* Make backup with datestamp first */
    sprintf(line, "%s.%ld", seenfile, now);
    fcopy(seenfile, line);
  }

  numSeenHosts = numSeenNicks = 0;

  if ( (hashnick = (itemaux **)calloc(HASHSIZE, sizeof(itemaux *))) &&
       (hashsite = (itemseen **)calloc(HASHSIZE, sizeof(itemseen *))) ) {

    if ( *seenfile && (f=fopen(seenfile, "r")) ) {

      while ( fgets(line, MAXLINE, f) ) {
        if (*line == ' ')
          SeenAddNick(p, &line[1]);
        else
          p = SeenAdd(line);
      }
      ret = TRUE;
      fclose(f);
    }
  }
  return (ret);
}

/* --- SeenSave --------------------------------------------------- */

void SeenSave(void)
{
  FILE *f;
  char tempfile[MAXLINE];
  int i;
  itemseen *ps;
  itemaux *pa;
  bool ok=TRUE;

  if(!*seenfile)
    return;

  if(!changedseenitem) {
    Logf(LOG, "No seen item changed, no save performed.");
    return;
  }
  changedseenitem = 0;

  /* Make backup first */
  sprintf(tempfile, "%s~", seenfile);
  rename(seenfile, tempfile);

  sprintf(tempfile, "%s_", seenfile);
  
  Logf(LOG, "Saving seen data \"%s\"", seenfile);

  /* Save seen data */
  if (f=fopen(tempfile, "w")) {
    for (ps = seenHead->link; ok && ps; ps = ps->link) {
      if ((ps->departure + SECINMONTH*seenmonths) >= now) {
        /*
         * Only save if the user departed within 'seenmonths' months ago!
         */
        if(0>fprintf(f, "%s %ld %d %s :%s\n", ps->host, ps->departure,
		     ps->type, (ps->by ? ps->by : "."),
		     (ps->msg ? ps->msg : ""))) {
	  ok=FALSE;
	  break;
	}
        for (pa = ps->first; ok && pa; pa = pa->link) {
	  if(!pa->departure ||
	     ((pa->departure + SECINMONTH*seenmonths) >= now)) {
	    /*
	     * Here too. We only save the nicks this person has used within
	     * the 'seenmonths' period. We shouldn't let old sins live on...
	     */
	    if(0>fprintf(f, " %s %ld %ld\n",
			 pa->nick, pa->count, pa->departure)) {
	      ok=FALSE;
	      break;
	    }
	  }
	}
      }
    }
    fclose(f);
    if(ok) {
      rename(tempfile, seenfile);
    }
  }
}

/* --- SeenCleanup ------------------------------------------------ */

void SeenCleanup(void)
{
  itemseen *ps, *tps;
  itemaux *pa, *tpa;

  if(!seenHead)
    return;

  SeenSave();

  /* Remember to free all allocated memory in structures */
  ps = seenHead->link;
  while (ps) {
    tps = ps->link;
    pa = ps->first;
    while (pa) {
      if (pa->nick) free(pa->nick);
      tpa = pa->link;
      free(pa);
      pa = tpa;
    }
    if (ps->usite) free(ps->usite);
    if (ps->host) free(ps->host);
    if (ps->by) free(ps->by);
    if (ps->msg) free(ps->msg);
    free(ps);
    ps = tps;
  }

  if (hashnick) free(hashnick);
  if (hashsite) free(hashsite);
}
