#ifdef RCS
static char rcsid[]="$Id: servfunc.c,v 1.13 1998/02/14 11:35:47 dast Exp $";
#endif
/******************************************************************************
 *                    Internetting Cooperating Programmers
 * ----------------------------------------------------------------------------
 *
 *  ____    PROJECT
 * |  _ \  __ _ _ __   ___ ___ _ __ 
 * | | | |/ _` | '_ \ / __/ _ \ '__|
 * | |_| | (_| | | | | (_|  __/ |   
 * |____/ \__,_|_| |_|\___\___|_|   the IRC bot
 *
 * Main authors:
 *  - Bjrn Reese <breese@imada.ou.dk>
 *  - Daniel Stenberg <Daniel.Stenberg@sth.frontec.se>
 *
 * Contributing authors (and ports):
 *  - Philippe Brand <phb@telesys-innov.fr>
 *  - Bjrn Stenberg <Bjorn.Stenberg@sth.frontec.se>
 *  - Andrew Gray <DarkDesigns@trader.com>
 *
 * All files in this archive are subject to the GNU General Public License.
 * 
 *               HTTP://www.imada.ou.dk/~breese/dancer.html
 *                FTP://ftp.netcom.com/pub/ra/raydar/Dancer
 *
 * $Source: /home/hardcore/dast/dancer/src/RCS/servfunc.c,v $
 * $Revision: 1.13 $
 * $Date: 1998/02/14 11:35:47 $
 * $Author: dast $
 * $State: Exp $
 * $Locker:  $
 *
 * ---------------------------------------------------------------------------
 * $Log: servfunc.c,v $
 * Revision 1.13  1998/02/14 11:35:47  dast
 * added boundary checks
 *
 * Revision 1.12  1996/12/12 18:59:34  dast
 * added NextServ()
 * better currentserv control
 * improved DelServ/()
 *
 * Revision 1.11  1996/12/06 15:57:24  dast
 * safer save
 *
 * Revision 1.10  1996/12/03 17:54:08  dast
 * added [DISABLED] text to 'serverlist' output
 *
 * Revision 1.9  1996/12/03 17:13:09  dast
 * the new .serv functionality
 *
 * Revision 1.8  1996/11/04 19:18:44  dast
 * now compiles without HAVE_LIBFPL
 *
 * Revision 1.7  1996/09/05 06:03:37  dast
 * added connect and disconnect hooks
 *
 * Revision 1.6  1996/08/30 11:29:13  dast
 * uses 'now'
 *
 * Revision 1.5  1996/08/21 11:17:31  dast
 * nothing
 *
 * Revision 1.4  1996/08/20 14:33:36  dast
 * DarkDesgn's changes
 *
 * Revision 1.3  1996/08/06 10:53:32  dast
 * Improved serverlist save
 *
 * Revision 1.2  1996/08/05 12:06:19  dast
 * major update
 *
 * Revision 1.1  1996/08/04 19:05:05  dast
 * Initial revision
 *
 *
 ****************************************************************************/

#include <stdarg.h>
#include <string.h>

#include "list.h"
#include "dancer.h"
#include "function.h"
#include "servfunc.h"
#include "fplrun.h"

extern time_t now;

extern char servfile[];
extern itemserv *currentserv;
extern char *errfrom;
extern char servername[];

itemserv *servHead;

static int servid=1;

itemserv *FindServ(char *name, long port)
{
  itemserv *s;
  int id=atoi(name);
  for (s=(itemserv *)Succ(servHead); s; s=(itemserv *)Succ(s)) {
    if((id && (id == s->s.id)) ||
       (!strcasecmp(name, s->s.name) && (port == s->s.port))) {
      return s; /* there! */
    }
  }
  return NULL;
}

itemserv *NextServ(void)
{
  itemserv *afterthis = currentserv;
  itemserv *s, *first;
  for (first = s =(itemserv *)Succ(servHead); s; s=(itemserv *)Succ(s)) {
    if(s->s.flags & SERV_DISABLED)
      continue;
    if(!afterthis)
      return s;
    else if(afterthis == s)
      afterthis = NULL; /* get the next */
  }
  return first; /* if nothing matched, return the first */
}


int AddServ(struct ServerSub *sub)
{
  itemserv *s;
  int retcode=TRUE;
  
  s = FindServ(sub->name, sub->port);
  if(!s) {
    s = (itemserv *)NewEntry(sizeof(itemserv));
    if(s) {
      AddTail(servHead, s);
      s->s=*sub;
      s->s.id = servid++;
      return FALSE;
    }
  }
  else if(s->s.flags&SERV_DISABLED &&
	  !(sub->flags&SERV_DISABLED) &&
	  !(sub->flags&SERV_INCONFIG)) {
    /*
     * The existing one is marked disabled. The new one is not, and it is
     * not from the .config file.
     * .config file entries are not allowed to undo disabled ones.
     */

    /*
     * The added one already exists, so we should return successful
     * but we should free the resources of this new struct since we
     * are reusing an old one.
     */
    s->s.flags &= ~SERV_DISABLED;
    retcode = FALSE; 
  }
  else if(sub->flags&SERV_INCONFIG) {
    s->s.flags |= SERV_INCONFIG; /* this exist in .config we know now */
  }
  
  /* free the resources */
  free(sub->name);
  if(sub->passwd)
    free(sub->passwd);
  if(sub->adder)
    free(sub->adder);
  if(sub->message)
    free(sub->message);

  return retcode;
}

int ManAddServ(char *name, long port, char *passwd, char *who, long flags)
{
  struct ServerSub sub;
  memset(&sub, 0, sizeof(struct ServerSub));
  sub.name = strdup(name);
  sub.port = port?port:IRCPORT;
  sub.passwd = passwd?strdup(passwd):NULL;
  sub.adder = who?strdup(who):NULL;
  sub.addtime = now;
  sub.flags = flags;
  return AddServ(&sub);
}

void ListServ(char *from, char *line)
{
  itemserv *s;
  char match[MIDBUFFER];
  strcpy(match, "*");
  sscanf(line, "%"MIDBUFFERTXT"s", match);
  for (s=(itemserv *)Succ(servHead); s; s=(itemserv *)Succ(s)) {
    if(Match(s->s.name, match)) {
      Sendf(from, "%d - %s %d%s%s", s->s.id, s->s.name, s->s.port,
            s->s.passwd?" <password>":"",
            s->s.flags&SERV_DISABLED?" [DISABLED]":"");
    }
  }
}

int SaveServ(void)
{
  itemserv *s;
  char tempname[BIGBUFFER];
  FILE *out;
  bool ok=TRUE;

  if(!servHead || !*servfile)
    return TRUE;

  sprintf(tempname, "%s~", servfile);
  out = fopen(tempname, "w");
  if(out) {
    if(0>fprintf(out, "# Dancer serverlist version: " VERSIONMSG "\n" )) {
      ok = FALSE;
    }
    for (s=(itemserv *)Succ(servHead);
         ok && s;
         s=(itemserv *)Succ(s)) {
      if(0>fprintf(out,
                   "%s:%d:%s\n"
                   ":%s\n"
                   "-%s\n"
                   "!%d %d %d %d %d %d\n",
                   s->s.name, s->s.port, s->s.passwd?s->s.passwd:"",
                   s->s.message?s->s.message:"",
                   s->s.adder?s->s.adder:"",
                   s->s.connect, s->s.disconnect, s->s.average,
                   s->s.numconnects, s->s.flags, s->s.addtime
                   )) {
        ok = FALSE;
        break;
      }
    }
    fclose(out);
    if(ok) {
      remove(servfile);
      rename(tempname, servfile);
    }
  }
  else
    return TRUE;
  return FALSE;
}

int LoadServ(void)
{
  FILE *in;
  char buffer[BIGBUFFER];
  struct ServerSub sub;
  char name1[BIGBUFFER];
  char name2[BIGBUFFER];
  int number[10];

  in = fopen(servfile, "r");
  if(in) {
    memset(&sub, 0, sizeof(struct ServerSub));
    while(fgets(buffer, BIGBUFFER, in)) {
      switch(buffer[0]) {
      case '#':
      case '\n':
        break;
      default:
        name2[0]=0;
        number[0]=IRCPORT;
        if(1<= sscanf(buffer, "%[^:]:%d:%[^\n]",
                      name1, &number[0], name2)) {
          sub.name=strdup(name1);
          sub.port=number[0];
          if(name2[0])
            sub.passwd = strdup(name2);
        }
        break;

      case ':':
        if(1== sscanf(buffer, ":%[^\n]", name1)) {
          sub.message=strdup(name1);
        }
        break;

      case '-':
        if(1== sscanf(buffer, "-%[^\n]", name1)) {
          sub.adder=strdup(name1);
        }
        break;
      case '!':
        if(6 == sscanf(buffer, "!%d %d %d %d %d %d",
                       &number[0], &number[1], &number[2],
                       &number[3], &number[4], &number[5])) {
          sub.connect = number[0];
          sub.disconnect = number[1];
          sub.average = number[2];
          sub.numconnects = number[3];
          sub.flags = number[4] & ~SERV_INCONFIG;
          sub.addtime = number[5];
        }
        AddServ(&sub);
        memset(&sub, 0, sizeof(struct ServerSub));
        break;
      }
    }
    fclose(in);
  }
  else
    return TRUE;
  return FALSE;
}

void FreeServ(void *v)
{
  itemserv *s=(itemserv *)v;
  if(s) {
    if(s->s.name)
      free(s->s.name);
    if(s->s.passwd)
      free(s->s.passwd);
    if(s->s.message)
      free(s->s.message);
    if(s->s.adder)
      free(s->s.adder);
  }
}

void ServCleanup(void)
{
  SaveServ();
  DestroyList((void **)&servHead, FreeServ);
}

int InitServ(void)
{
  extern itemlist *serverHead;
  itemlist *p;
  char server[BIGBUFFER];
  char serverpasswd[BIGBUFFER];

  servHead = CreateList(sizeof(itemserv));
  if(servHead) {
    /*
     * First, we get the servers from the .serv file
     */
    LoadServ(); 

    /*
     * Now, we convert the old-style serverlist:
     */

    for (p=Succ(serverHead); p; p=Succ(p)) {
      int serverport=IRCPORT;
      serverpasswd[0]=0;
      sscanf((char *)p->ptr, "%[a-zA-Z0-9.-]%*[ :]%d%*[ :]%[^\n]",
	     server, &serverport, serverpasswd);
      ManAddServ(server, serverport, serverpasswd[0]?serverpasswd:
		 NULL, NULL, SERV_INCONFIG);
    }
  }
  return TRUE;
}

int DelServ(char *name, long port)
{
  itemserv *s;
  s = FindServ(name, port);
  if(s) {
    if(s->s.flags & SERV_INCONFIG)
      /* this is entered in the .config file and therefore we cannot
	 remove this for real, just mark it disabled */
      s->s.flags |= SERV_DISABLED;
    else 
      DeleteEntry(servHead, s, FreeServ); /* remove */
    return FALSE;
  }
  return TRUE;
}

int ManDelServ(char *from, char *line)
{
  char server[SEMIBUFFER];
  long port=IRCPORT;
  if(sscanf(line, "%"SEMIBUFFERTXT"s %d", server, &port)) {
    if(currentserv &&
       !strcasecmp(server, currentserv->s.name) &&
       port == currentserv->s.port) {
      Send(errfrom, GetText(msg_cant_delete_current_server));
      return TRUE;
    }
    if(DelServ(server, port)) {
      Sendf(errfrom, GetText(msg_could_not_find_server), server, port);
      return TRUE;
    }
    Sendf(from, GetText(msg_server_removed), server, port);
    SaveServ();
  }
  else
    return TRUE;
  return FALSE;
}

void StatServ(char *from, int options, char *server, long port)
{
  itemserv *s;
  if(!server[0]) {
    s = currentserv;
    if(!s) {
      Sendf(errfrom, GetText(msg_server_not_in_list), servername);
      return;
    }
  }
  else {
    s = FindServ(server, port);
    if(!s) {
      Sendf(errfrom, GetText(msg_could_not_find_server), server, port);
      return;
    }
  }
  /* the switch makes duplicate flags get ignored, only one of them set
     will make a change */
  switch(options) {
  case SERV_ENABLED:
    s->s.flags &= ~SERV_DISABLED;
    break;
  case SERV_DISABLED:
    s->s.flags |= SERV_DISABLED;
    break;
  }
  Sendf(from, "%s port %d %s", s->s.name, s->s.port,
        s->s.passwd?"<passwd>":"");
  Sendf(from, "%s last connect %s ago", currentserv==s?"<current>":"",
        s->s.connect?TimeAgo(s->s.connect):"ages");
  Sendf(from, "last disconnect %s ago, average connect time %s",
        s->s.disconnect?TimeAgo(s->s.disconnect):"ages",
        SecsToString(s->s.average));
  Sendf(from, "number of connects: %d", s->s.numconnects);
  Sendf(from, "State:%s",
	s->s.flags&SERV_DISABLED?" [DISABLED]":" normal");

}

void ConnectServ(itemserv *serv)
{
  currentserv = serv;
  if(serv) {
#ifdef HAVE_LIBFPL
    if(runfpl(RUN_CONNECT, serv->s.name, FPLRUN_PRE))
      return;
#endif
    serv->s.connect = now;
    serv->s.numconnects++;
    SaveServ(); /* store server info */
    runfpl(RUN_CONNECT, serv->s.name, FPLRUN_POST);
  }
}

void DisconnectServ(char *fmt, ...)
{
  va_list args;
  char buf[BIGBUFFER];
  
  va_start(args, fmt);
  vsprintf(buf, fmt, args);
  va_end(args);
#ifdef HAVE_LIBFPL
  if(runfpl(RUN_DISCONNECT, buf, FPLRUN_PRE))
    return;
#endif

  if(currentserv) {

    if(currentserv->s.message)
      free(currentserv->s.message);
    currentserv->s.message = strdup(buf);
    currentserv->s.disconnect = now;

    /* calculate the new average connect time */
    currentserv->s.average = ((currentserv->s.average *
			       (currentserv->s.numconnects-1)) +
			      (currentserv->s.disconnect -
			       currentserv->s.connect)) /
				 currentserv->s.numconnects;
  }
  runfpl(RUN_DISCONNECT, buf, FPLRUN_POST);
}

