#ifdef RCS
static char rcsid[]="$Id: tell.c,v 2.3 1998/02/14 11:36:41 dast Exp $";
#endif
/******************************************************************************
 *                    Internetting Cooperating Programmers
 * ----------------------------------------------------------------------------
 *
 *  ____    PROJECT
 * |  _ \  __ _ _ __   ___ ___ _ __ 
 * | | | |/ _` | '_ \ / __/ _ \ '__|
 * | |_| | (_| | | | | (_|  __/ |   
 * |____/ \__,_|_| |_|\___\___|_|   the IRC bot
 *
 * Main authors:
 *  - Bjrn Reese <breese@imada.ou.dk>
 *  - Daniel Stenberg <Daniel.Stenberg@sth.frontec.se>
 *
 * Contributing authors (and ports):
 *  - Philippe Brand <phb@telesys-innov.fr>
 *  - Bjrn Stenberg <Bjorn.Stenberg@sth.frontec.se>
 *
 * All files in this archive are subject to the GNU General Public License.
 * 
 *               HTTP://www.imada.ou.dk/~breese/dancer.html
 *                FTP://ftp.netcom.com/pub/ra/raydar/Dancer
 *
 * $Source: /home/hardcore/dast/dancer/src/RCS/tell.c,v $
 * $Revision: 2.3 $
 * $Date: 1998/02/14 11:36:41 $
 * $Author: dast $
 * $State: Exp $
 * $Locker:  $
 *
 * ---------------------------------------------------------------------------
 * $Log: tell.c,v $
 * Revision 2.3  1998/02/14 11:36:41  dast
 * added boundary checks
 *
 * Revision 2.2  1997/11/05 22:02:51  dast
 * updated!
 *
 * Revision 2.1  1997/05/28 06:25:07  dast
 * Fixed yet another telldel bug
 * Incorporated the news system in TELLME
 *
 * Revision 1.12  1997/04/28 16:48:04  dast
 * Saves the tell-list after each TELL command
 * Uses safe-save methods
 *
 * Revision 1.11  1997/02/10 19:41:36  dast
 * several improvements
 * '
 *
 * Revision 1.10  1997/01/07 16:34:43  dast
 * checks for totally illegal patterns before allowing a tell
 *
 * Revision 1.9  1996/12/03 17:14:37  dast
 * better error check in cleanup
 *
 * Revision 1.8  1996/11/27 20:35:26  dast
 * now uses bigger buffer when reading TELLs from file
 *
 * Revision 1.7  1996/08/30 11:29:13  dast
 * breese's listfix
 *
 * Revision 1.6  1996/08/06 07:07:09  dast
 * Improved the hostpattern checks for TellNotify() and TellMe()
 *
 * Revision 1.5  1996/07/10 16:15:00  dast
 * Added the source header.
 *
 *
 *****************************************************************************/

#include <stdio.h>
#include <string.h>

#include "dancer.h"
#include "function.h"
#include "user.h"
#include "transfer.h"
#include "tell.h"
#include "list.h"
#include "news.h"

/* --- Global ----------------------------------------------------- */

extern time_t now;

extern char tellfile[], nickname[];
extern itemident *current;
extern int tellmonths;

itemtell *tellHead = NULL;

int numTell = 0;
int newid = 0;

/* ---------------------------------------------------------------- */

itemtell *AddTell(char *sender, char *receiver, char *msg, time_t when)
{
  itemtell *p;
  char *ptr = strrchr(receiver, '!');
  if(ptr) {
    /* In case we have a nick-part in the receiver-pattern, we scan for the
       '!' and points to the rest of the string instead */
    receiver = ++ptr;
  }

  if(!strchr(receiver, '@'))
    /* illegal pattern, we can't use this one */
    return NULL;

  if (IsPrefix(receiver))
    /* Skip prefix */
    ++receiver;

  p = (itemtell *)NewEntry(sizeof(itemtell));
  if (p) {
    AddTail(tellHead, p);
    p->sender    = strdup(sender);
    p->receiver  = strdup(receiver);
    p->rcvdomain = Userdomain(receiver);
    p->message   = strdup(msg);
    p->when      = when;
    p->id        = newid++;
    numTell++;
  }
  return p;
}

void FreeTell(void *v)
{
  itemtell *p;

  p = (itemtell *)v;
  if (p) {
    if (p->sender) free(p->sender);
    if (p->receiver) free(p->receiver);
    if (p->rcvdomain) free(p->rcvdomain);
    if (p->message) free(p->message);
    numTell--;
  }
}

inline void ReportTell(char *from, itemtell *t)
{
  Sendf(from, "%s %s \"%s\" %s %s", t->sender, GetText(msg_said),
        t->message, TimeAgo(t->when), GetText(msg_ago));
}

/* --- TellNotify ------------------------------------------------- */

extern itemclient *client;
extern bool chat;

void TellNotify(itemguest *pc)
{
  itemtell *p, *pp;
  int mcnt = 0;
  long news=0;
  itemnews *n;
  char newstr[MINIBUFFER]="";
  char msgstr[MINIBUFFER]="";

  client = NULL;
  chat = FALSE;

  for (p=Succ(tellHead); p; p=Succ(p)) {
    if ( Match(p->receiver, pc->ident->userdomain) ) {
      pp = p;
      mcnt++;
    }
    else {
      itemuser *user=pc->ident->user;
      if(user) {
        itemlist *list;
        for(list = Succ(user->domainhead); list; list = Succ(list)) {
          if( Match( p->receiver, list->ptr)) {
            pp = p;
            mcnt++;
            break;
          }
        }
      }
    }
  }
  pc->msgs = mcnt;

  if(pc->ident->user) {
    current = pc->ident; /* silly fix */
    news = NewsRead(pc->ident->user->newsid, &n);
  }

  /* News and Tells combined: */
  switch (mcnt + news) {
  case 0: /* none */
    break;
  case 1: /* tell immediately */
    if(mcnt) {
      ReportTell(pc->ident->nick, pp);
      pc->msgs = 0;
      DeleteEntry(tellHead, pp, FreeTell);
    }
    else {
      NewsReport(pc->ident);
    }
    break;
  default: /* TELLME to tell you */
    if(mcnt)
      sprintf(msgstr, "%d %s ", mcnt,
              (mcnt > 1) ? GetText(msg_msgs) : GetText(msg_msg));
    if(news)
      sprintf(newstr, " %d %s ", (int)news, GetText(msg_news_items));

    SendNickf(pc->ident->nick, GetText(msg_i_have_tells),
              mcnt?msgstr:"",
              (news&&mcnt)?GetText(msg_and):"",
              news?newstr:"",
              nickname);
    break;
  }
}

itemtell *TellNext(char *from, itemguest *w, itemtell *t, bool all)
{
  ReportTell(from, t);
  w->msgs--;
  if (!all && w->msgs > 0) {
    Sendf(from, GetText(msg_you_have_more), w->msgs,
          (w->msgs > 1) ? GetText(msg_msgs) : GetText(msg_msg));
  }
  return (DeleteEntry(tellHead, t, FreeTell));
}

/* --- TellMe ----------------------------------------------------- */

void TellMe(char *from, char *host, bool all)
{
  itemtell *t;
  int cnt = 0;
  itemguest *w;

  w = current->guest;
  if (w) {
    t = Succ(tellHead);
    while (t) {
      if ( Match(t->receiver, host) ) {
        t = TellNext(from, w, t, all);
        if(!all)
          return;
        cnt++;
        continue;
      }
      else if(w->ident->user) {
        itemlist *list;
        bool found=FALSE;
        for(list = Succ(w->ident->user->domainhead); list;
            list = Succ(list)) {
          if( Match( t->receiver, list->ptr)) {
            t = TellNext(from, w, t, all);
            if(!all)
              return;
            cnt++;
            found=TRUE;
            break; /* for ('found' ensures a 'continue' on the while loop) */
          }
        }
        if(found)
          continue; /* while */
      }
      t = Succ(t);
    }
    if (!cnt) {
      if(current->user &&
         NewsRead(current->user->newsid, NULL))
        NewsReport(current);
      else
        Send(from, GetText(msg_no_msgs_or_news));
    }
  } else
    Send(from, GetText(msg_please_join_channel));
}

/* --- TellAdd ---------------------------------------------------- */

void TellAdd(char *from, char *sender, char *receiver, char *msg)
{
  itemtell *p;

  if (receiver && (p=AddTell(sender, receiver, msg, now)) ) {
    TellSave();
    Sendf(from, GetText(msg_message_stored), p->rcvdomain, p->id);
  }
  else {
    if(!receiver)
      receiver = "<empty>";
    Sendf(from, GetText(msg_error_message_pattern), receiver);
  }
}

/* --- TellDel ---------------------------------------------------- */

void TellDel(char *from, char *fromhost, char *which)
{
  itemtell *p;
  long id;
  char buf[MIDBUFFER];
  char *ptr;
  char *fromp;
  char *top;
  bool deleted=FALSE;

  id=strtol(which, &ptr, 10);

  if ( which != ptr ) {  /* Find number */
    p = Succ(tellHead);

    fromp = Userdomain(fromhost);
      
    while (p) {
      /* Not p->sender, but Userdomain of host part */
      if (p->id == id) {
        sscanf(p->sender, "%*[^!]%*[!]%[^\n]", buf);
        top = Userdomain(buf);

        if( Match(fromp, top)) {
          Sendf(from, GetText(msg_had_been_removed), p->id);
          p = DeleteEntry(tellHead, p, FreeTell);
          /* p is now pointing on next item */
          deleted = TRUE;

        } else
          Sendf(from, GetText(msg_not_your_messageid), p->id);

        free(top);
        break;
      }
      p = Succ(p);
    }
    if(!p && !deleted)
      Send(from, GetText(msg_no_such_id));

    free(fromp);
  }
}

/* --- TellList --------------------------------------------------- */

void TellList(char *from, char *host)
{
  itemtell *p;
  int cnt = 0;
  char *shost;

  for (p=Succ(tellHead); p; p=Succ(p)) {
    if ( (shost=strchr(p->sender, '!')) ) {
      if ( Match(host, shost + 1) ) {
        if (!cnt)
          Send(from, GetText(msg_you_have_stored_these));
        Sendf(from, "%d: %s %s %s", p->id, p->receiver,
              TimeAgo(p->when), GetText(msg_ago));
        cnt++;
      }
    }
  }
  if (!cnt)
    Send(from, GetText(msg_tell_list_empty));
}

/* --- TellInit --------------------------------------------------- */

void TellInit(void)
{
  FILE *f;
  char buf[MAXLINE];
  char sender[MINIBUFFER], receiver[MINIBUFFER];
  char msg[BIGBUFFER];
  time_t when;

  tellHead = CreateList(sizeof(itemtell));

  numTell = 0;
  if ( *tellfile && (f=fopen(tellfile, "r")) ) {
    if ( fgets(buf, MAXLINE, f) ) {
      newid = atoi(buf);
      while (fgets(buf, MAXLINE, f)) {
        if (4 == sscanf(buf, "%"MINIBUFFERTXT"s %"MINIBUFFERTXT"s %d %[^\n]",
			sender, receiver, &when, msg) )
          AddTell(sender, receiver, msg, when);
      }
    }
    fclose(f);
  }
}

/* --- TellSave() ---------------------------------------------- */

void TellSave(void)
{
  FILE *f;
  itemtell *p;
  char tempfile[MINIBUFFER];
  bool ok= TRUE;

  /* perform a "safe" save */
  sprintf(tempfile, "%s~", tellfile);

  if(tellHead) {
    if ( *tellfile && (f=fopen(tempfile, "w")) ) {
      if(0>fprintf(f, "%d\n", newid))
        ok = FALSE;
      for (p=Succ(tellHead); p; p=Succ(p)) {
	/* Ignore messages older than two months */
	if ((now - p->when) < tellmonths*SECINMONTH) {
	  if(0>fprintf(f, "%s %s %d %s\n", p->sender, p->receiver, (int)p->when,
                       p->message))
            ok = FALSE;
        }
      }
      fclose(f);
      if(ok)
        rename(tempfile, tellfile);
    }
  }
}

/* --- TellCleanup ------------------------------------------------ */

void TellCleanup(void)
{
  TellSave();
  DestroyList((void **)&tellHead, FreeTell);
}
